package fr.inra.pappso.xtandempipeline;

import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import fr.inra.moulon.proticport.model.Controller;
import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSourceSet;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptide;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptideSet;
import fr.inra.pappso.xtandempipeline.grouping.HashProtein;
import fr.inra.pappso.xtandempipeline.grouping.HashProteinSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSet;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpMsSample;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class XtandemPipelineSession {

	private static XtandemPipelineSession session = null;

	private static final Logger logger = Logger
			.getLogger(XtandemPipelineSession.class);

	private HashSampleScanSet sampleScanSet = new HashSampleScanSet();
	private IdentificationDataSourceSet dataSourceSet = new IdentificationDataSourceSet();
	private HashPeptideSet peptideSet = new HashPeptideSet();
	private HashProteinSet proteinSet = new HashProteinSet();

	private ArrayList<Identification> identificationList = new ArrayList<Identification>();

	private config conf;

	private DataType_Base dataType;

	private String dataSource = "flat_file";

	private Controller ppController;

	private URL ppUrl;

	private String ppLogin;

	private String ppPassword;

	private XpipFile currentXpipFile;

	private XtandemPipelineSession() {
		conf = new config();
		dataType = new DataType_Base(conf);
	}

	static public XtandemPipelineSession getInstance() {
		if (session == null) {
			session = new XtandemPipelineSession();
		}
		return (session);
	}

	public HashSampleScan getHashSampleScanInstance(int scan,
			GrpMsSample grpMsSample) {
		return sampleScanSet.getInstance(scan, grpMsSample);
	}

	public HashPeptide getHashPeptideInstance(HashSampleScan hashScan,
			String get_LI_sequence) throws MSMSException {
		return peptideSet.getInstance(hashScan, get_LI_sequence);
	}

	public HashProtein getHashProteinInstance(String accession) {
		return proteinSet.getInstance(accession);
	}

	public IdentificationDataSource getIdentificationDataSourceInstance(
			String Source) throws MSMSException {
		return dataSourceSet.getInstance(Source);
	}

	public ArrayList<Identification> getIdentificationList() {
		return identificationList;
	}

	public config getConfig() {
		return this.conf;
	}

	public DataType_Base getDataTypeBase() {
		return this.dataType;
	}

	public void setDataTypeBase(DataType_Base type) {
		this.dataType = type;
	}

	public void clearIdentificationResult() {
		logger.info("Clearing session");
		this.dataSource = "flat_file";
		ppController = null;
		sampleScanSet.clear();
		identificationList.clear();
		peptideSet.clear();
		proteinSet.clear();
		identificationList = new ArrayList<Identification>();
		sampleScanSet = new HashSampleScanSet();
		peptideSet = new HashPeptideSet();
		proteinSet = new HashProteinSet();

		this.dataType = new DataType_Base(conf);

		System.gc();

	}

	public void setPROTICportController(URL url, String login, String password) {
		setPROTICportDataSource();
		this.ppUrl = url;
		if (login == null) {
			this.ppLogin = null;
			this.ppPassword = null;
		} else {
			if (!login.equals(ppLogin)) {
				this.ppPassword = null;
			}
			this.ppLogin = login;
			if (password != null) {
				this.ppPassword = password;
			}
		}
	}

	public Controller getPROTICportControllerSwt(Shell parentShell)
			throws MSMSException {

		if (this.isPROTICportDataSource() == false) {
			return null;
		}
		if (ppController == null) {
			logger.debug("ppController null");
			if (ppPassword == null) {
				logger.debug("ppController null");
				Shell shell = new Shell(parentShell);
				MessageBox messageBox = new MessageBox(shell, SWT.ABORT);
				messageBox
						.setMessage("Il faut demander le mot de passe à l'utilisateur");
				messageBox.open();
				try {
					this.ppController = fr.inra.moulon.proticport.client.ClientProticPortFactory
							.getFactoryInstance().getController(ppUrl, ppLogin,
									ppPassword);
				} catch (ProticPortException e) {
					String error = "problem connecting to PROTICport :\n"
							+ e.getMessage();
					logger.error(error);
					throw new MSMSException(error);
				}
			} else {
				ppController = getPROTICportController();
			}
		}
		return ppController;
	}

	public Controller getPROTICportController() throws MSMSException {
		if (this.isPROTICportDataSource() == false) {
			return null;
		}
		if (ppUrl == null) {
			return null;
		}
		if (ppController == null) {
			try {
				if (ppLogin != null) {
					if (ppPassword == null) {
						return null;
					}
				}
				this.ppController = fr.inra.moulon.proticport.client.ClientProticPortFactory
						.getFactoryInstance().getController(ppUrl, ppLogin,
								ppPassword);
			} catch (ProticPortException e) {
				String error = "problem connecting to PROTICport :\n"
						+ e.getMessage();
				logger.error(error);
				throw new MSMSException(error);
			}
			return this.ppController;
		}
		return ppController;

	}

	public boolean isPROTICportDataSource() {
		if (this.dataSource.equals("PROTICport")) {
			logger.debug("PROTICport datasource");
			return true;
		}
		return false;
	}

	public void setPROTICportDataSource() {
		this.dataSource = "PROTICport";
	}

	public String getIdentificationDataSourceKey(
			IdentificationDataSource sampleFile) throws MSMSException {
		return this.dataSourceSet.getKey(sampleFile);
	}

	public void setCurrentXpipFile(XpipFile xpipFile) {
		logger.debug(" file : " + xpipFile.getAbsolutePath());

		this.currentXpipFile = xpipFile;
	}

	public XpipFile getCurrentXpipFile() {
		return this.currentXpipFile;
	}

	public void setHttpAuthorization(URLConnection uc) throws MSMSException {
		if (getPROTICportController() == null) {
			// return uc;
		} else {
			if ((this.ppLogin != null) && (this.ppPassword != null)) {
				String userpass = this.ppLogin + ":" + this.ppPassword;
				String basicAuth = "Basic "
						+ new String(new Base64().encode(userpass.getBytes()));
				uc.setRequestProperty("Authorization", basicAuth);
			}
		}
	}

	public void forgetExistingXpipFile() {
		this.currentXpipFile = null;
	}
}
