package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;

import org.apache.log4j.Logger;

import com.compomics.mascotdatfile.util.interfaces.MascotDatfileInf;
import com.compomics.mascotdatfile.util.mascot.Query;
import com.compomics.mascotdatfile.util.mascot.enumeration.MascotDatfileType;
import com.compomics.mascotdatfile.util.mascot.factory.MascotDatfileFactory;

import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.sax_parse.Mascot;

public class IdentificationMascotFile extends IdentificationDataFile {

	private static final Logger logger = Logger
			.getLogger(IdentificationMascotFile.class);

	public IdentificationMascotFile(File mascotFile) throws MSMSException {
		super(mascotFile);
	}

	@Override
	public Identification loadIdentification(
			Identification previousIdentification) throws Exception {
		Mascot mascot = new Mascot(previousIdentification, this);
		mascot.readAndParseMascotFile();
		previousIdentification = mascot.getIdentification();
		return previousIdentification;
	}

	@Override
	public Identification loadPhosphoIdentification(
			Identification previousIdentification) throws Exception {
		Mascot mascot = new Mascot(previousIdentification, this);
		mascot.setPhosphoFilter(true);
		mascot.readAndParseMascotFile();
		previousIdentification = mascot.getIdentification();
		return previousIdentification;
	}

	@Override
	public boolean loadMSMSSpectra(Peptide pep) throws Exception {
		// TODO Make MascotMSMS parser
		MascotDatfileInf mascot = MascotDatfileFactory.create(
				this.file.getAbsolutePath(), MascotDatfileType.INDEX);
		Query q = mascot.getQuery(pep.get_scan());
		if (q != null) {
			pep.setMz_values(q.getMZArray());
			pep.setIntensity_values(q.getIntensityArray());
			return true;
		}
		return false;
	}

	@Override
	public void setScanCorrespondance(int fixedScanNum,
			int scanNumAsWrittenInDataSource) {

	}

	@Override
	public Integer getScanCorrespondance(int scanNumber) {
		return null;
	}

	@Override
	public void writeOdsInformations(OdsDocWriterInterface odsTable)
			throws MSMSException {
		String message = " writeOdsInformations not implemented for IdentificationMascotFile objects";
		logger.warn(message);

	}

	@Override
	public String getSampleName() throws MSMSException {
		return this.getName().replaceAll("\\.dat$", "");
	}
}
