package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.moulon.proticport.model.Controller;
import fr.inra.moulon.proticport.model.entities.MsRun;
import fr.inra.moulon.proticport.model.entities.Spectrum;
import fr.inra.moulon.proticport.model.exceptions.ProticDataNotFoundException;
import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;

public class IdentificationProticPortSource implements IdentificationDataSource {
	private static final Logger logger = Logger
			.getLogger(IdentificationProticPortSource.class);
	private Long msRunDbId;
	private Boolean cacheExists = null;

	public IdentificationProticPortSource(Long msRunDbId) {
		this.msRunDbId = msRunDbId;
	}

	@Override
	public void writeXpipPeptideSourceAttribute(XMLStreamWriter writer)
			throws Exception {
		writer.writeAttribute("sample_file", "" + this.msRunDbId);
	}

	@Override
	public boolean loadMSMSSpectra(Peptide pep) throws Exception {
		logger.debug("PP");
		Controller ppController = XtandemPipelineSession.getInstance()
				.getPROTICportController();
		if (ppController == null) {
			throw new MSMSException("PROTICport connexion failed");
		} else {
			Spectrum spectrum = ppController.getProticPortFactory()
					.newSpectrum();
			logger.debug("getting spectrum");
			try {
				spectrum.get(pep.get_scan(), ppController);
				pep.setMz_values(spectrum.getMzArr(ppController));
				pep.setIntensity_values(spectrum.getIntenArr(ppController));
			} catch (ProticDataNotFoundException e) {
				throw new MSMSException("Spectrum not found in PROTICdb");
			}
		}
		return true;
	}

	@Override
	public Identification loadIdentification(
			Identification previousIdentification) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Identification loadPhosphoIdentification(
			Identification previousIdentification) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String getName() {
		return this.msRunDbId.toString();
	}

	@Override
	public int compareTo(IdentificationDataSource toCompare) {
		IdentificationProticPortSource PPSourcetoCompare = (IdentificationProticPortSource) toCompare;
		return this.msRunDbId.compareTo(PPSourcetoCompare.msRunDbId);
	}

	@Override
	public boolean equals(Object toCompare) {
		IdentificationProticPortSource PPSourcetoCompare = (IdentificationProticPortSource) toCompare;
		return this.msRunDbId.equals(PPSourcetoCompare.msRunDbId);
	}

	@Override
	public boolean exists() {
		if (cacheExists != null) {
			return this.cacheExists;
		}
		Controller ppController;
		try {
			ppController = XtandemPipelineSession.getInstance()
					.getPROTICportController();
			if (ppController == null) {
				throw new MSMSException("PROTICport connexion failed");
			} else {
				MsRun msRun = ppController.getProticPortFactory().newMsRun();
				logger.debug("getting msRun");
				try {
					msRun.get(this.msRunDbId, ppController);
					cacheExists = true;
					return true;
				} catch (ProticDataNotFoundException e) {
					String message = "MsRun not found in PROTICdb";
					logger.error(message);
					// throw new MSMSException(message);
				} catch (ProticPortException e) {
					String message = "ProticPortException " + e.getMessage();
					logger.error(message);
				}
			}
		} catch (MSMSException e1) {
			String message = "MSMSException " + e1.getMessage();
			logger.error(message);
		}
		cacheExists = false;
		return false;
	}

	@Override
	public void setScanCorrespondance(int fixedScanNum,
			int scanNumAsWrittenInDataSource) {

	}

	@Override
	public Integer getScanCorrespondance(int scanNumber) {
		return null;
	}

	@Override
	public void writeOdsInformations(OdsDocWriterInterface odsTable)
			throws MSMSException {
		String message = " writeOdsInformations not implemented for IdentificationProticPortSource objects";
		logger.warn(message);
	}

	@Override
	public String getSampleName() throws MSMSException {
		return this.getName();
	}
}
