package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashProtein;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpProtMatch;

public class Match implements Comparable<Match>, GrpProtMatch {

	private static final Logger logger = Logger.getLogger(Match.class);

	private HashMap<HashSampleScan, Peptide> peptides_match;
	// Par defaut la table est vide, il faut la remplir à postériori

	private ArrayList<Peptide> peptides_order;

	private Protein protein_match;

	private boolean isvalidate;

	@Override
	public int compareTo(Match p) {
		int k = 0;
		k = p.getValidSpectraNumber() - this.getValidSpectraNumber();
		if (k != 0)
			return (k);
		int i = p.peptides_match.size() - this.peptides_match.size();
		if (i != 0)
			return (i);
		return (this.protein_match.compareTo(p.get_protein_match()));
	}

	public Match() {
		this.peptides_match = new HashMap<HashSampleScan, Peptide>();
		this.peptides_order = new ArrayList<Peptide>();
		this.protein_match = new Protein();
		this.isvalidate = true;
	}

	// @Override
	// protected void finalize() throws Throwable {
	// logger.debug("finalize Match");
	// }

	public void add_peptide_match(Peptide P) {
		// if (!this.peptides_match.containsKey(P.get_sample_scan_cle()))
		// this.peptides_match.put(P.get_sample_scan_cle(), P);
		this.peptides_order.add(P);
	}

	public void IndexAndOrderPeptideListe() throws MSMSException {
		this.peptides_match.clear();
		Collections.sort(peptides_order);
		for (Peptide pep : peptides_order) {
			if (!this.peptides_match.containsKey(pep.getHashSampleScan())) {
				this.peptides_match.put(pep.getHashSampleScan(), pep);
			} else {
				if (this.peptides_match.get(pep.getHashSampleScan())
						.get_evalue() > pep.get_evalue()) {
					throw new MSMSException("OUPS problems of peptide order"
							+ pep.get_sample_scan_sequencepeptide_cle());
				}
			}
		}
	}

	public ArrayList<Peptide> get_all_peptides() {
		return (peptides_order);
	}

	@Override
	public Collection<GrpPeptide> getGrpPeptides() throws MSMSException {
		ArrayList<GrpPeptide> peps = new ArrayList<GrpPeptide>();
		for (Peptide pep : peptides_order) {
			if (pep.isValid())
				peps.add(pep);
		}
		return (peps);
	}

	public ArrayList<Peptide> get_valid_peptides() throws MSMSException {
		ArrayList<Peptide> peps = new ArrayList<Peptide>();
		for (Peptide pep : peptides_order) {
			if (pep.isValid())
				peps.add(pep);
		}
		return (peps);
	}

	public void set_protein_match(Protein P) {
		this.protein_match = P;
	}

	public Protein get_protein_match() {
		return (this.protein_match);
	}

	public MsRunSet get_sample_matchs() throws MSMSException {
		MsRunSet samples = new MsRunSet();
		for (Peptide pep : peptides_match.values()) {
			samples.add(pep.getMsRun());
		}
		return (samples);
	}

	/**
	 * count number of spectra coming from one MsRun (MS sample)
	 *
	 * @param samp
	 *            the sample to count spectra from
	 * @return
	 * @throws MSMSException
	 */
	private int getValidSpectraNumberIn(MsRun samp) throws MSMSException {
		int i = 0;
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				if (pep.getMsRun().equals(samp)) {
					i++;
				}
			}
		}
		return (i);
	}

	/**
	 * count number of spectra in this match
	 *
	 * @return integer number of spectra
	 * @throws MSMSException
	 */
	public int getValidSpectraNumber() {
		int i = 0;
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				i++;
			}
		}

		return (i);
	}

	/**
	 * compute the Protein Abundance Index (PAI)
	 *
	 * @return
	 */
	public double getPAI() {
		// compute PAI
		double value = (double) this.getValidSpectraNumber()
				/ (double) this.get_protein_match()
						.getNumberOfTrypticPeptidesForPAI();

		return value;
	}

	/**
	 * compute the Protein Abundance Index (PAI) within an MsRun
	 *
	 * @return
	 */
	public double getPAIbyMsRun(MsRun samp) throws MSMSException {
		// compute PAI
		double value = (double) this.getValidSpectraNumberIn(samp)
				/ (double) this.get_protein_match()
						.getNumberOfTrypticPeptidesForPAI();

		return value;
	}

	public double getEmPAI() {
		// compute emPAI, Ishihama 2005
		double value = Math.pow(10.0, getPAI()) - (double) 1.0;

		return value;
	}

	public double getEmPAIbyMsRun(MsRun samp) throws MSMSException {
		// compute emPAI, Ishihama 2005
		double value = Math.pow(10.0, getPAIbyMsRun(samp)) - (double) 1.0;

		return value;
	}

	public int get_unique_valid_peptide_number_to(MsRun samp)
			throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				if (pep.getMsRun().equals(samp)) {
					sequence.add(pep.getSequence());
				}
			}
		}
		return (sequence.size());
	}
/*
	public ArrayList<Peptide> get_peptide_order_inclus_redon() {
		Collections.sort(peptides_order);
		return (peptides_order);
	}
*/
	public ArrayList<Peptide> get_peptide_order() {
		ArrayList<Peptide> peptides = new ArrayList<Peptide>(
				peptides_match.values());
		Collections.sort(peptides);
		return (peptides);
	}

	public int get_unique_valid_peptide_number() throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				sequence.add(pep.getSequence());
			}
		}
		return (sequence.size());
	}

	public int get_unique_valid_peptide_number_to_sequencemodifs()
			throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				sequence.add(pep.get_LI_sequence_modifs());
			}
		}
		return (sequence.size());
	}

	public int get_coverage() throws MSMSException {
		// Unknown sequence
		if (this.protein_match.get_sequence().length() == 0) {
			return 0;
		}

		HashMap<Integer, Boolean> seq = this.get_coverage_hashtable();

		int k = 0;
		for (Boolean ST : seq.values()) {
			if (ST)
				k++;
		}

		float cov = (float) k
				/ (this.protein_match.get_sequence().length() + 1) * 100;
		// System.out.println(cov);
		int cov_temp = (int) cov;
		// System.out.println(cov_temp);
		return (cov_temp);
	}

	public HashMap<Integer, Boolean> get_coverage_hashtable()
			throws MSMSException {
		HashMap<Integer, Boolean> seq = new HashMap<Integer, Boolean>();
		if (this.protein_match.get_sequence().length() == 0) {
			return (seq);
		}

		for (int i = 0; i < this.protein_match.get_sequence().length(); i++) {
			int i_po = i + 1;
			seq.put(i_po, false);
		}
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValid()) {
				for (int j = pep.get_start(); j <= pep.get_stop(); j++) {
					seq.put(j, true);
				}
			}
		}
		return seq;
	}

	public void setIsvalidate(boolean isvalidate) {
		this.isvalidate = isvalidate;
	}

	public boolean isIsvalidate() {
		return isvalidate;
	}

	@Override
	public HashProtein getHashProtein() throws MSMSException {
		return this.protein_match.getHashProtein();
	}

	public ArrayList<HashSampleScanSetPhospho> getHashSampleScanSetPhosphoList()
			throws Exception {
		// TODO
		ArrayList<HashSampleScanSetPhospho> phosphoSiteList = new ArrayList<HashSampleScanSetPhospho>();
		// TODO : determine phosphosite positions
		PhosphoIslandSet islands = new PhosphoIslandSet(this);
		for (PhosphoIsland island : islands) {
			phosphoSiteList.add(new HashSampleScanSetPhospho(this, island
					.getPhosphoPositions()));
		}
		logger.debug("getHashSampleScanSetPhosphoList end "
				+ phosphoSiteList.size());
		return phosphoSiteList;
	}

	@Override
	public GrpPeptide getBestPeptideToHashSampleScan(HashSampleScan hash)
			throws MSMSException {
		Peptide pep = this.peptides_match.get(hash);
		if (pep == null) {
			logger.error("Hastable of peptide is not indexed");
			this.IndexAndOrderPeptideListe();
		}
		return (this.peptides_match.get(hash));
	}

	@Override
	public Collection<GrpPeptide> getGrpPeptidesSharingHashSampleScan(
			HashSampleScan hash) throws MSMSException {
		ArrayList<GrpPeptide> peps = new ArrayList<GrpPeptide>();
		for (Peptide pep : this.peptides_order) {
			if (pep.getHashSampleScan().equals(hash) && pep.isValid()) {
				peps.add(pep);
			}
		}
		return (peps);
	}

	public ArrayList<Peptide> getAllValidPeptideToHashSampleScan(
			HashSampleScan hash) throws MSMSException {
		ArrayList<Peptide> peps = new ArrayList<Peptide>();
		for (Peptide pep : this.peptides_order) {
			if (pep.getHashSampleScan().equals(hash) && pep.isValid()) {
				peps.add(pep);
			}
		}
		return (peps);
	}

	public HashMap<HashSampleScan, Peptide> getHashSampleScanToPeptide() {
		return (peptides_match);
	}

	@Override
	public Protein getGrpProtein() {
		return protein_match;
	}

	@Override
	public ArrayList<Peptide> getValidPeptideInPhosphoIsland(
			HashSampleScanSetPhospho hashPhospho) throws MSMSException {
		ArrayList<Peptide> peps = new ArrayList<Peptide>();
		for (HashSampleScan sampleScanPhospho : hashPhospho
				.getHashSampleScanList()) {
			peps.addAll(this
					.getAllValidPeptideToHashSampleScan(sampleScanPhospho));
		}
		return (peps);
	}

	public int getNumberOfPeptides() {
		return peptides_match.size();
	}

	public void tryToDeterminePeptidePositions() throws MSMSException {
		logger.debug("tryToDeterminePeptidePositions begin");
		Protein protein = this.get_protein_match();
		for (Peptide pep : peptides_order) {
			protein.tryToPlacePeptide(pep);
		}
	}
}
