/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.File;
import java.util.HashMap;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerMzxmlGetPeptideSpectra;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerMzxmlSummary;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;

public class MzxmlFile {
	private static final Logger logger = Logger.getLogger(MzxmlFile.class);

	private File mzxmlFile = null;

	public MzxmlFile(String fileName) {
		logger.debug(fileName);
		mzxmlFile = new File(fileName);
	}

	public String getName() {
		return mzxmlFile.getName();
	}

	public boolean exists() {
		return mzxmlFile.exists();
	}

	public String getAbsolutePath() {
		return mzxmlFile.getAbsolutePath();
	}

	public void writeOdsInformations(OdsDocWriterInterface odsTable)
			throws MSMSException {
		logger.debug("writeOdsInformations in mzXML begin");

		if (!this.mzxmlFile.exists()) {
			try {
				// odsTable.writeLine();
				// odsTable.writeCell("mzXML file NOT FOUND");
				// odsTable.writeCell(this.file.getAbsolutePath());
			} catch (Exception e) {
				String message = "ERROR writing ODS in writeOdsXtandemInformations "
						+ e.getMessage();
				logger.error(message);
				throw new MSMSException(message);
			}
		} else {
			HandlerMzxmlSummary handler = new HandlerMzxmlSummary();
			try {
				new parse_file(handler, this.mzxmlFile);
			} catch (Exception e) {
				String message = "ERROR parsing file "
						+ this.mzxmlFile.getName() + " " + e.getMessage();
				logger.warn(message);
				return;
				// throw new MSMSException(message);
			}

			try {
				HashMap<Integer, Long> msLevelCounter = handler
						.getMsLevelCounter();
				HashMap<Integer, Double> msLevelIntensitySum = handler
						.getMsLevelIntensitySum();

				odsTable.writeLine();
				for (Integer msLevel : msLevelCounter.keySet()) {
					odsTable.writeCell("MS level " + msLevel);
				}
				odsTable.writeLine();
				for (Integer msLevel : msLevelCounter.keySet()) {
					odsTable.writeCell(msLevelCounter.get(msLevel));
				}
				odsTable.writeLine();
				for (Integer msLevel : msLevelIntensitySum.keySet()) {
					odsTable.writeCell("TIC mean in MS " + msLevel);
				}
				odsTable.writeLine();
				for (Integer msLevel : msLevelIntensitySum.keySet()) {
					odsTable.writeCell((msLevelIntensitySum.get(msLevel) / (new Double(
							msLevelCounter.get(msLevel)))));
				}
				odsTable.writeLine();
				odsTable.writeCell("rt min");
				odsTable.writeCell("rt max");
				odsTable.writeLine();
				odsTable.writeCell(handler.getRtMin());
				odsTable.writeCell(handler.getRtMax());

			} catch (Exception e) {
				String message = "ERROR writing ODS in writeOdsInformations (mzXml file) "
						+ e.getMessage();
				logger.error(message);
				throw new MSMSException(message);
			}
		}
	}

	public void loadMSMSSpectra(Peptide pep) throws MSMSException {

		if (!this.mzxmlFile.exists()) {
			String message = "ERROR in loadMSMSSpectra mzXML file "
					+ this.getAbsolutePath() + " does not exists";
			logger.error(message);
			throw new MSMSException(message);

		} else {
			HandlerMzxmlGetPeptideSpectra handler = new HandlerMzxmlGetPeptideSpectra(
					pep);
			try {
				new parse_file(handler, this.mzxmlFile);
			} catch (Exception e) {
				String message = "ERROR parsing file "
						+ this.mzxmlFile.getName() + " " + e.getMessage();
				logger.error(message);
				throw new MSMSException(message);
			}
		}
	}
}
