package fr.inra.pappso.xtandempipeline.grouping;

import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpMsSample;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpProtMatch;

public class HashSampleScanSetProt extends HashSampleScanSet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7558622484317846088L;
	/**
	 * 
	 */
	private static final Logger logger = Logger
			.getLogger(HashSampleScanSetProt.class);

	protected GrpProtMatch match;

	public HashSampleScanSetProt(GrpProtMatch match) throws MSMSException {
		this.setGrpProtMatch(match);
		for (GrpPeptide peptide : match.getGrpPeptides()) {
			HashSampleScan hashSample = peptide.getHashSampleScan();
			if (hashSample == null) {
				String message = "hashSample is null";
				logger.error(message);
			}
			this.add(hashSample);
		}
	}

	@Override
	public void clear() {
		match = null;
		super.clear();
	}

	@Override
	public String toString() {
		return match.getGrpProtein().getAccession();
	}

	protected HashSampleScanSetProt() {
		match = null;
	}

	@Override
	public int compareTo(HashSampleScanSet arg0) {
		// TODO Auto-generated method stub
		int i = this.size() - arg0.size();
		if (i != 0)
			return i;
		return this
				.getGrpProtMatch()
				.getGrpProtein()
				.compareTo(
						((HashSampleScanSetProt) arg0).getGrpProtMatch()
								.getGrpProtein());
	}

	@Override
	public boolean equals(Object toCompare) {
		if (super.equals(toCompare)) {
			HashSampleScanSetProt incoming = (HashSampleScanSetProt) toCompare;
			if (incoming.getGrpProtMatch().getGrpProtein().getAccession()
					.equals(this.match.getGrpProtein().getAccession())) {
				return (true);
			}
			return (false);
		}
		return false;
	}

	protected void setGrpProtMatch(GrpProtMatch match) throws MSMSException {
		if (match == null) {
			String message = "Macth null";
			logger.error(message);
			throw new MSMSException(message);
		}
		this.match = match;
	}

	public GrpProtMatch getGrpProtMatch() {

		return match;
	}

	public HashKey4GroupsSet getHashKey4GroupsSet() throws Exception {
		HashPeptideSet hashPeptideSet = new HashPeptideSet();

		for (GrpPeptide peptide : match.getGrpPeptides()) {
			HashPeptide p = peptide.getHashPeptide();
			if (p == null) {
				String message = "hashPeptide is null";
				logger.error(message);
				throw new Exception(message);
			}

			hashPeptideSet.add(p);
		}

		return hashPeptideSet;
	}

	public int getUniquePeptideHashSampleScan() throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (HashSampleScan hash : this) {
			GrpPeptide pep = this.match.getBestPeptideToHashSampleScan(hash);
			sequence.add(pep.getSequence());
		}
		return (sequence.size());
	}

	public int getTotalHashSampleScan() {
		return (this.size());
	}

	public int getTotalHashSampleScantoMsRun(GrpMsSample sample) {
		int count = 0;
		for (HashSampleScan hash : this) {
			if (hash.getGrpMsSample().equals(sample))
				count++;
		}
		return (count);
	}

	public int getUniqueHashPeptideSampleScantoMsRun(GrpMsSample sample)
			throws MSMSException {
		HashSet<String> sequence = new HashSet<String>();
		for (HashSampleScan hash : this) {
			GrpPeptide pep = this.match.getBestPeptideToHashSampleScan(hash);
			if (pep.getHashSampleScan().getGrpMsSample().equals(sample)) {
				sequence.add(pep.getSequence());
			}
		}
		return (sequence.size());
	}

	public HashSet<GrpMsSample> getGrpMsSampleSet() {
		HashSet<GrpMsSample> grpMsSampleList = new HashSet<GrpMsSample>();
		for (HashSampleScan hash : this) {
			grpMsSampleList.add(hash.getGrpMsSample());
		}
		return (grpMsSampleList);
	}

	/**
	 * compute the Protein Abundance Index (PAI)
	 * 
	 * @return
	 */
	public double getPAI() {
		// compute PAI
		double value = (double) this.getTotalHashSampleScan()
				/ (double) this.match.getGrpProtein()
						.getNumberOfTrypticPeptidesForPAI();

		return value;
	}

	/**
	 * compute the Protein Abundance Index (PAI) within an MsRun
	 * 
	 * @return
	 */
	public double getPAIbyMsRun(GrpMsSample samp) throws MSMSException {
		// compute PAI

		double value = (double) getTotalHashSampleScantoMsRun(samp)
				/ (double) this.match.getGrpProtein()
						.getNumberOfTrypticPeptidesForPAI();

		return value;
	}

	public double getEmPAI() {
		// compute emPAI, Ishihama 2006
		double value = Math.pow(10.0, getPAI()) - (double) 1.0;

		return value;
	}

	public double getEmPAIbyMsRun(MsRun samp) throws MSMSException {
		// compute emPAI, Ishihama 2006
		double value = Math.pow(10.0, getPAIbyMsRun(samp)) - (double) 1.0;

		return value;
	}
}
