package fr.inra.pappso.xtandempipeline.output;

/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;

public class FastaOutputStream {
	private static final Logger logger = Logger
			.getLogger(FastaOutputStream.class);

	protected OutputStreamWriter out = null;
	protected HashSet<Protein> alreadyThere = new HashSet<Protein>();

	public FastaOutputStream(OutputStream out)
			throws UnsupportedEncodingException {
		this.out = new OutputStreamWriter(out, "UTF8");
	}

	protected FastaOutputStream() {
	}

	public void write(Protein protein) throws MSMSException {
		if (this.out == null) {
			logger.error("out is null");
		}
		if (protein == null) {
			logger.error("protein is null");
		}

		if (alreadyThere.add(protein)) {
			try {
				this.out.write(get_fasta(protein));
			} catch (IOException e) {
				String message = "IOException writing protein "
						+ protein.get_description() + " to fasta output\n"
						+ e.getMessage();
				logger.error(message);
				throw new MSMSException(message, e);
			}
		}

	}

	private String get_fasta(Protein protein) {
		String fasta = ">" + protein.get_description() + "\n";
		String sequence = protein.get_sequence();
		for (int i = 0; i < sequence.length(); i += 60) {
			if (i + 60 < sequence.length())
				fasta += sequence.substring(i, i + 60) + "\n";
			else {
				fasta += sequence.substring(i, sequence.length()) + "\n";
			}
		}
		return (fasta);
	}

	public void close() throws IOException {
		if (this.out == null) {
			logger.error("out is null");
		}
		this.out.close();
	}

}
