package fr.inra.pappso.xtandempipeline.output;

/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

//import com.sun.xml.internal.txw2.output.IndentingXMLStreamWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class PepnovoMlWriter {
	private static final Logger logger = Logger
			.getLogger(PepnovoMlWriter.class);

	private XMLStreamWriter writer;

	protected PepnovoMlWriter(XMLStreamWriter writer) throws Exception {
		logger.info("Save project ...");

		this.writer = writer;

		writer.writeStartElement("tandem_to_pepnovo");
	}

	public void write(Identification ident) throws XMLStreamException,
			MSMSException {
		for (MsRun msrun : ident.getMsRunSet()) {
			writer.writeStartElement("sample");
			writer.writeAttribute("file", msrun.getSampleName());

			for (Group group : ident.getGrouping().getGroupList()) {
				this.print_group_information_to_sample(group, msrun);
			}
			// sample
			writer.writeEndElement();
		}

	}

	private void print_group_information_to_sample(Group group, MsRun sample)
			throws XMLStreamException, MSMSException {
		for (SubGroup sg : group.getSubGroupSet()) {
			HashSampleScanSetProt hashProt = sg.getHashSampleScanSetProtList()
					.get(0);
			Match top = (Match) hashProt.getGrpProtMatch();
			// StringBuilder S = new StringBuilder("");
			int totalNumber = hashProt.getTotalHashSampleScantoMsRun(sample);
			if (totalNumber > 0) {
				writer.writeStartElement("protein");
				writer.writeAttribute("nb",
						Utils.getPappsoSubGroupId(group, sg));
				// S.append("<protein nb=\"" + this.get_subgroup_number(group,
				// sg)
				// + "\" ");
				writer.writeAttribute("desc", top.get_protein_match()
						.get_description());
				// S.append("desc=\"" +
				// top.get_protein_match().get_description()
				// + "\" ");
				writer.writeAttribute("pep", "" + totalNumber);
				// S.append("pep=\"" + totalNumber + "\" >" + "\n");
				writer.writeStartElement("sequence");
				writer.writeCharacters(top.get_protein_match().get_sequence());
				// S.append("<sequence>" +
				// top.get_protein_match().get_sequence()
				// + "</sequence>" + "\n");
				writer.writeEndElement();
				for (HashSampleScan hash : hashProt) {
					if (hash.getGrpMsSample().equals(sample)) {
						writer.writeStartElement("scan");
						writer.writeAttribute("nb", "" + hash.getScan());
						// S.append("<scan nb=\"" + hash.getScan() + "\" />"
						// + "\n");
						writer.writeEndElement();
					}
				}

				writer.writeEndElement();
				// S.append("</protein>" + "\n");
				// out.write(S.toString());
			}
		}

	}

	public void close() throws XMLStreamException {

		writer.writeEndDocument();

		writer.flush();
		writer.close();

	}
}
