/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.TreeSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class OdsPhosphoPeptideOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPhosphoPeptideOutput.class);

	protected OdsDocWriterInterface odsTable;

	protected OdsPhosphoPeptideOutput() {

	}

	public OdsPhosphoPeptideOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("phosphopeptides");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPhosphoPeptideOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("phosphopeptides");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPhosphoPeptideOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group ID Peptide ID Sequence Modifs Number of phosphorylation
			// Best Position in sequence All Position Charge MH+ theo subgroups
			// ids number of spectra

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("Sub-group ID");
			odsTable.writeCell("Phosphopeptide ID");
			odsTable.writeCell("SequenceLI");
			odsTable.writeCell("Modifs");
			odsTable.writeCell("Number of phosphorylations (phosphosites)");
			odsTable.writeCell("Best Position in sequence");
			odsTable.writeCell("All positions");
			odsTable.writeCell("Charges");
			odsTable.writeCell("MH+ theo");
			odsTable.writeCell("Number of proteins");
			// odsTable.writeCell("Protein ids");
			odsTable.writeCell("Number of spectra");
			// odsTable.writeCell("Delta-ppm");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				PeptideMassSet pepMassSet = new PeptideMassSet();

				HashMap<PeptideMass, HashSet<Peptide>> hashPeptideMass2Peptides = pepMassSet
						.numberingPhospho(group);

				logger.debug("pepMassSet size : " + pepMassSet.size());

				logger.debug("pepMassSet numbering");
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					for (PeptideMass pepMass : pepMassSet) {
						SubGroup sgPhosphoPep = pepMass.getSubGroupSet()
								.iterator().next();
						if (sgPhosphoPep.getNumber().equals(sg.getNumber())) {
							this.writeOnePhosphoPeptideMass(ident, group, sg,
									pepMass,
									hashPeptideMass2Peptides.get(pepMass));
						}
					}
				}
			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOnePhosphoPeptideMass(Identification ident, Group group,
			SubGroup sg, PeptideMass pepMass, HashSet<Peptide> peptideSet)
			throws MSMSException {
		try {
			logger.debug("writeOnePhosphoPeptideMass begin " + group + pepMass);
			odsTable.writeLine();
			Peptide bestPeptide = pepMass.getBestEvaluePeptide();

			// phospho subgroups are based on sequence : one pepMass should have
			// only one subgroup, and all prot in this subgroup shares the same
			// sample scan numbers
			/*
			 * ArrayList<SubGroup> sgList = pepMass.getSubGroupSet()
			 * .getSubGroupList(); HashSampleScanSetPhospho
			 * hashSampleScanSetPhospho = (HashSampleScanSetPhospho) sgList
			 * .get(0).getHashSampleScanSetProtList().get(0);
			 */
			HashSampleScanSetPhospho hashSampleScanSetPhospho = (HashSampleScanSetPhospho) sg
					.getHashSampleScanSetProtList().get(0);
			// odsTable.writeCell("Group ID");
			odsTable.writeCell(Utils.getPappsoGroupId(group));
			odsTable.writeCell(Utils.getPappsoSubGroupId(group, sg));
			// odsTable.writeCell("Peptide ID");
			odsTable.writeCell(Utils.getPappsoPhosphoPeptideMassId(group,
					pepMass));
			// odsTable.writeCell("Sequence");
			odsTable.writeCell(pepMass.getSequenceLi());
			// odsTable.writeCell("Modifs");
			odsTable.writeCell(pepMass.getMhDiff());
			// odsTable.writeCell("Number of phosphorylations");
			Collection<GrpPhosphoModif> phosphoModifs = bestPeptide
					.getGrpPhosphoModifs();
			odsTable.writeCell(phosphoModifs.size());

			TreeSet<Integer> obsPhosphoPos = new TreeSet<Integer>();
			for (GrpPhosphoModif phosphoModif : phosphoModifs) {
				Modifs modif = (Modifs) phosphoModif;
				obsPhosphoPos.add(modif.get_posi());
			}
			// odsTable.writeCell("Best Position in sequence");
			odsTable.writeCell(Utils.collectionToString(obsPhosphoPos));

			HashSet<GrpPhosphoModif> phosphoAllModifs = new HashSet<GrpPhosphoModif>();
			for (Peptide peptide : peptideSet) {
				phosphoAllModifs.addAll(peptide.getGrpPhosphoModifs());
			}

			phosphoModifs = hashSampleScanSetPhospho
					.getAllPhosphoModif(pepMass);
			obsPhosphoPos = new TreeSet<Integer>();
			for (GrpPhosphoModif phosphoModif : phosphoModifs) {
				Modifs modif = (Modifs) phosphoModif;
				obsPhosphoPos.add(modif.get_posi());
			}
			// odsTable.writeCell("All positions");
			// TreeSet<Integer> posi = phospho.getAllPositionIncluderedon(hash);
			odsTable.writeCell(Utils.collectionToString(obsPhosphoPos));

			TreeSet<Integer> obsCharges = new TreeSet<Integer>();
			for (Peptide peptide : peptideSet) {
				obsCharges.add(peptide.get_charge());
			}
			// odsTable.writeCell("Charges");
			odsTable.writeCell(Utils.collectionToString(obsCharges));

			// odsTable.writeCell("MH+ theo");
			odsTable.writeCell(bestPeptide.get_mhplus_theo());

			// SubGroupSet sgList = pepMass.getSubGroupSet();
			// odsTable.writeCell("Number of subgroups");
			odsTable.writeCell(sg.getHashSampleScanSetProtSet().size());

			/*
			 * TreeSet<String> sgIds = new TreeSet<String>(); for (SubGroup sg :
			 * sgList) { sgIds.add(Utils.getPappsoSubGroupId(group, sg)); } //
			 * odsTable.writeCell("Subgroup ids");
			 * odsTable.writeCell(Utils.collectionToString(sgIds));
			 */

			// odsTable.writeCell("Number of spectra");
			odsTable.writeCell(pepMass.getHashSampleScanSet().size());

			logger.debug("writeOnePeptideMass end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOnePeptideMass "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOnePeptideMass "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
