package fr.inra.pappso.xtandempipeline.output;

import java.util.ArrayList;

import javax.xml.stream.XMLStreamException;
//import com.sun.xml.internal.txw2.output.IndentingXMLStreamWriter;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.moulon.proticport.model.Controller;
import fr.inra.pappso.xtandempipeline.XtandemPipelineMain;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;
import fr.inra.pappso.xtandempipeline.thread.XtpProcess;

public class XpipWriter {
	private static final Logger logger = Logger.getLogger(XpipWriter.class);

	private ArrayList<Identification> idents;

	private XMLStreamWriter writer;

	private XtpProcess xtpProcess;

	protected XpipWriter(XMLStreamWriter writer) throws Exception {
		logger.info("Save project ...");

		this.writer = writer;

		writer.writeStartElement("xtandem_pipeline");
		writer.writeAttribute("version", XtandemPipelineMain.version);
	}

	private void write_result_to(Identification ident) throws Exception {
		writer.writeStartElement("identification");

		writer.writeStartElement("modifs_list_mass");
		for (Modifs mod : ident.get_modifs_liste().get_modifs_order()) {
			writer.writeEmptyElement("modifs_mass");
			writer.writeAttribute("modvalue", "" + mod.get_modvalue());
		}
		writer.writeEndElement();

		writer.writeStartElement("samples");
		for (MsRun samp : ident.getMsRunSet()) {
			writer.writeEmptyElement("sample");
			writer.writeAttribute("value", samp.getSampleName());
		}

		writer.writeEndElement();
		writer.writeStartElement("match_list");
		for (Match m : ident.get_protein_liste().getMatchOrder())
			this.write_match_to(m, ident);
		writer.writeEndElement();
		// on ferme l'ident
		writer.writeEndElement();
		// writer.flush();
	}

	private void write_match_to(Match m, Identification ident) throws Exception {
		writer.writeStartElement("match");

		writer.writeAttribute("validate", "" + m.isIsvalidate());

		writer.writeAttribute("PAI", "" + m.getPAI());
		writer.writeAttribute("emPAI", "" + m.getEmPAI());

		writer.writeStartElement("protein");
		writer.writeAttribute("peptide_number", "" + m.getNumberOfPeptides());
		writer.writeAttribute("evalue", "" + m.get_protein_match().get_evalue());
		writer.writeAttribute("URL", m.get_protein_match().getDatabase()
				.getDatabasePath());
		writer.writeAttribute("description", m.get_protein_match()
				.get_description());
		// sample evalue
		for (MsRun sample : m.get_protein_match().get_hash_sample_evalue()
				.keySet()) {
			writer.writeEmptyElement("protein_evalue");
			writer.writeAttribute("evalue", ""
					+ m.get_protein_match().get_hash_sample_evalue()
							.get(sample));
			writer.writeAttribute("sample", sample.getSampleName());
			// writer.writeCharacters(endline);
		}

		writer.writeStartElement("sequence");
		writer.writeCharacters(m.get_protein_match().get_sequence());
		writer.writeEndElement();
		writer.writeEndElement();

		writer.writeStartElement("peptide_list");
		for (Peptide pep : m.get_all_peptides()) {
			pep.writeXpip(writer);
			// this.write_peptide_to(pep);
		}
		writer.writeEndElement();
		// on ferme le match
		writer.writeEndElement();
		// on compte les match
		xtpProcess.add_current_progress();
	}

	protected void close() throws XMLStreamException {
		writer.writeEndDocument();
	}

	protected void write(ArrayList<Identification> identifications,
			DataType_Base datatype, XtpProcess p) throws Exception {
		this.idents = identifications;
		this.xtpProcess = p;

		// <DataSource>
		// <PROTICport
		// url="http://uln.moulon.inra.fr/~langella/proticdev/web_view/proticport/"
		// user="langella" />
		// </DataSource>
		if (XtandemPipelineSession.getInstance().isPROTICportDataSource()) {
			writer.writeStartElement("DataSource");
			writer.writeEmptyElement("PROTICport");
			Controller ppc = XtandemPipelineSession.getInstance()
					.getPROTICportController();
			writer.writeAttribute("url", ppc.getProticPortUrl().toString());
			writer.writeAttribute("user", ppc.getUser().getLogin());
			writer.writeEndElement();
		}
		writer.writeEmptyElement("information");
		writer.writeAttribute("Data_Type", datatype.getType());
		int count = 0;
		for (Identification ident : idents)
			count += ident.get_protein_liste().size();
		writer.writeAttribute("match_number", "" + count);

		writer.writeEmptyElement("filter_params");
		writer.writeAttribute("pep_evalue", ""
				+ datatype.getConf().get_peptide_evalue());
		writer.writeAttribute("prot_evalue", ""
				+ datatype.getConf().get_protein_evalue());
		writer.writeAttribute("pep_number", ""
				+ datatype.getConf().get_peptide_number());
		writer.writeAttribute("filter_to_all", ""
				+ datatype.getConf().get_protein_filter_to_complete());
		writer.writeAttribute("database_filter", datatype.getConf()
				.get_database_filter().getAbsolutePath());

		writer.writeStartElement("identifications");
		for (Identification ident : idents)
			this.write_result_to(ident);
		writer.writeEndElement();
		// on ferme le root
		writer.writeEndElement();
		writer.writeEndDocument();
		writer.flush();
		writer.close();

		logger.info("Save project finish");
	}

}
