package fr.inra.pappso.xtandempipeline.sax_parse;

/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.thread.XtandemCondorAnalysis;

public class HandlerHTCondorQueue extends SaxBaseHandler {

	private static final Logger logger = Logger
			.getLogger(HandlerHTCondorQueue.class);
	private Object inName;
	private Integer currentProcId;
	private Integer currentJobStatus;
	private String currentRemoteHost;
	private String currentLastRemoteHost;
	private int junexpanded = 0;
	private int jidle = 0;
	private int jrunning = 0;
	private int jremoved = 0;
	private int jcompleted = 0;
	private int jheld = 0;
	private int jsubmission_error = 0;
	private XtandemCondorAnalysis xtandemCondorAnalysis;
	private boolean empty = true;

	public HandlerHTCondorQueue(XtandemCondorAnalysis xtandemCondorAnalysis)
			throws MSMSException {
		super();
		this.xtandemCondorAnalysis = xtandemCondorAnalysis;
	}

	@Override
	public void startDocument() throws SAXException {
		this.empty = true;
	}

	@Override
	public void endDocument() throws SAXException {
		int reallyCompletedJobs = 0;
		if (this.empty) {
			reallyCompletedJobs = xtandemCondorAnalysis.getCondorJobNumber();
		} else {
			int totalJobs = junexpanded + jidle + jrunning + jremoved
					+ jcompleted + jheld + jsubmission_error;
			int diffJobs = xtandemCondorAnalysis.getCondorJobNumber()
					- totalJobs;
			reallyCompletedJobs = jcompleted + diffJobs;
		}

		xtandemCondorAnalysis.setJobCompleted(junexpanded, jidle, jrunning,
				jremoved, reallyCompletedJobs, jheld, jsubmission_error);

		String statusMessage = "" + junexpanded + " unexpanded jobs\n";
		statusMessage += jidle + " idle jobs\n";
		statusMessage += jrunning + " running jobs\n";
		statusMessage += jremoved + " removed jobs\n";
		statusMessage += reallyCompletedJobs + " completed jobs\n";
		statusMessage += jheld + " held jobs\n";
		statusMessage += jsubmission_error + " submission_errors\n";

		logger.debug(statusMessage);
		xtandemCondorAnalysis.setStatusMessage(statusMessage);

	}

	// <c>
	// <a n="ProcId"><i>0</i></a>
	public void startElementc(String nameSpaceUri, Attributes attrs) {
		this.empty = false;
		currentProcId = null;
		currentJobStatus = null;
		currentRemoteHost = null;
		currentLastRemoteHost = null;

	}

	public void startElementa(String nameSpaceUri, Attributes attrs) {
		// logger.debug("startElementgroup begin");
		// <group label="performance parameters" type="parameters">
		this.inName = attrs.getValue("n");

	}

	public void endElementi(String nameSpaceUri, String data)
			throws MSMSException {
		if (inName.equals("ProcId")) {
			this.currentProcId = new Integer(data);
			// logger.debug(currentProcId);
		} else if (inName.equals("JobStatus")) {
			// <a n="JobStatus"><i>2</i></a>
			this.currentJobStatus = new Integer(data);
			// logger.debug(currentProcId);
		}

	}

	public void endElements(String nameSpaceUri, String data)
			throws MSMSException {
		if (inName.equals("RemoteHost")) {
			// <a n="RemoteHost"><s>slot1@proteus3</s></a>
			this.currentRemoteHost = data;
			// logger.debug(currentProcId);
		} else if (inName.equals("LastRemoteHost")) {
			// <a n="LastRemoteHost"><s>slot1@proteus4</s></a>
			this.currentLastRemoteHost = data;
			// logger.debug(currentProcId);
		}

	}

	public void endElementa(String nameSpaceUri, String data) {
		this.inName = "";
	}

	public void endElementc(String nameSpaceUri, String data)
			throws MSMSException {
		/*
		 * 0 Unexpanded U 1 Idle I 2 Running R 3 Removed X 4 Completed C 5 Held
		 * H 6 Submission_err E
		 */
		if (currentJobStatus == 0) {
			this.junexpanded += 1;
		} else if (currentJobStatus == 1) {
			this.jidle += 1;
		} else if (currentJobStatus == 2) {
			this.jrunning += 1;
		} else if (currentJobStatus == 3) {
			this.jremoved += 1;
		} else if (currentJobStatus == 4) {
			this.jcompleted += 1;
		} else if (currentJobStatus == 5) {
			this.jheld += 1;
		} else if (currentJobStatus == 6) {
			this.jsubmission_error += 1;
		}

		logger.debug("currentRemoteHost:" + this.currentRemoteHost
				+ " currentLastRemoteHost" + this.currentLastRemoteHost + " "
				+ currentJobStatus);
	}

}
