package fr.inra.pappso.xtandempipeline.sax_parse;

/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;

import fr.inra.moulon.utils.Base64;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;

public class HandlerMzxmlGetPeptideSpectra extends SaxBaseHandler {
	private static final Logger logger = Logger
			.getLogger(HandlerMzxmlSummary.class);
	private Boolean isMzXml = null;
	private Integer currentScanNum;
	private Peptide peptide;
	private boolean isPrecision64;
	private boolean bigendian;
	private String pair_order;
	private double[] intenArr;
	private double[] mzArr;

	public HandlerMzxmlGetPeptideSpectra(Peptide pep) throws MSMSException {
		this.peptide = pep;
	}

	@Override
	protected void startUnknownElement(String nameSpaceUri, String sName,
			String qName, Attributes attrs) throws MSMSException,
			SAXNotRecognizedException {
		if (this.isMzXml == null) {
			logger.debug("startUnknownElement " + qName);
			if (qName.equals("mzXML")) {
				this.isMzXml = true;
			} else {
				this.isMzXml = false;
				throw new SAXNotRecognizedException("this is not an mzXML file");
			}
		}
	}

	public void startElementmzXML(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementmzXML begin");
		this.isMzXml = true;
	}

	@Override
	public void endDocument() throws SAXException {
	}

	public void startElementscan(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		// logger.debug("startElementscan begin ");
		/*
		 * <scan polarity="+" highMz="1829.35" totIonCurrent="1.07252e+008"
		 * peaksCount="732" scanType="Full" retentionTime="PT300.03S"
		 * basePeakIntensity="1.08706e+006" msLevel="1" num="1" lowMz="400.631"
		 * basePeakMz="462.039"> <peaks byteOrder="network" pairOrder="m/z-int"
		 * precision="32"></peaks> </scan>
		 */

		currentScanNum = new Integer(attrs.getValue("num"));
	}

	public void startElementpeaks(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		// writer.writeAttribute("precision", "32");
		// writer.writeAttribute("byteOrder", "network"); == > true
		// writer.writeAttribute("pairOrder", "m/z-int");

		if (peptide.get_scan() == currentScanNum) {

			// compressionType="zlib"
			if (attrs.getValue("compressionType") != null) {
				String message = "compressionType "
						+ attrs.getValue("compressionType")
						+ " is not taken into account";
				logger.warn(message);
				throw new MSMSException(message);
			}
			isPrecision64 = false;
			bigendian = true;
			pair_order = "m/z-int";
			if (attrs.getValue("precision") != null) {
				if (attrs.getValue("precision").equals("32")) {
					isPrecision64 = false;
				} else {
					isPrecision64 = true;
				}
			}
			if (attrs.getValue("byteOrder") != null) {
				if (attrs.getValue("byteOrder").equals("network")) {
					bigendian = true;
				} else {
					bigendian = false;
				}
			}
			if (attrs.getValue("pairOrder") != null) {
				if (attrs.getValue("pairOrder").equals("m/z-int")) {
					pair_order = "m/z-int";
				} else {
					pair_order = "int-m/z";
				}
			}
		}
	}

	public void endElementpeaks(String nameSpaceUri, String data)
			throws MSMSException {

		if (peptide.get_scan() == currentScanNum) {
			logger.debug("setBase64Peaks");
			setBase64Peaks(data, isPrecision64, bigendian, pair_order);
			peptide.setIntensity_values(intenArr);
			peptide.setMz_values(mzArr);
		}
	}

	/**
	 * Convert the byte array to an int starting from the given offset.
	 *
	 * @param b
	 *            The byte array
	 * @param offset
	 *            The array offset
	 * @return The integer
	 */
	public static int byteArrayToInt(byte[] b, int offset) {
		// logger.debug("byteArrayToInt begin " + b);
		int value = 0;
		for (int i = 0; i < 4; i++) {
			int shift = (4 - 1 - i) * 8;
			value += (b[i + offset] & 0x000000FF) << shift;
		}
		// logger.debug("byteArrayToInt value " + value);
		return value;
	}

	/**
	 * Convert the byte array to an int.
	 *
	 * @param b
	 *            The byte array
	 * @return The integer
	 */
	public static final int byteArrayToInt(byte[] value) {
		return byteArrayToInt(value, 0);
	}

	public static final float[] Bytes2ArrayFloat(byte[] decodedArr,
			boolean bigendian) {
		int length = decodedArr.length / 4;
		float[] result = new float[length];
		byte[] swap = new byte[4];
		int k = 0;
		for (int i = 0; i < decodedArr.length; i += 4, k++) {
			for (int j = 0; j < 4; j++) {
				if (bigendian) {
					swap[j] = decodedArr[i + j];
				} else {
					swap[3 - j] = decodedArr[i + j];
				}
			}
			result[k] = Float.intBitsToFloat(byteArrayToInt(swap));
		}
		return result;
	}

	public static long toLong(byte[] data) {

		if (data == null || data.length != 8)
			return 0x0;

		// ----------

		return (long) (

		// (Below) convert to longs before shift because digits

		// are lost with ints beyond the 32-bit limit

		(long) (0xff & data[0]) << 56 |

		(long) (0xff & data[1]) << 48 |

		(long) (0xff & data[2]) << 40 |

		(long) (0xff & data[3]) << 32 |

		(long) (0xff & data[4]) << 24 |

		(long) (0xff & data[5]) << 16 |

		(long) (0xff & data[6]) << 8 |

		(long) (0xff & data[7]) << 0

		);

	}

	public static double toDouble(byte[] data) {

		if (data == null || data.length != 8) {
			// logger.debug("data length " + data.length);
			return 0x0;
		}
		// ---------- simple:
		// logger.debug("data " + data);

		return Double.longBitsToDouble(toLong(data));
	}

	public static double[] toDoubleA(byte[] data) {

		if (data == null)
			return null;

		// ----------

		if (data.length % 8 != 0)
			return null;

		double[] dbls = new double[data.length / 8];

		for (int i = 0; i < dbls.length; i++) {

			dbls[i] = toDouble(new byte[] {

			data[(i * 8)],

			data[(i * 8) + 1],

			data[(i * 8) + 2],

			data[(i * 8) + 3],

			data[(i * 8) + 4],

			data[(i * 8) + 5],

			data[(i * 8) + 6],

			data[(i * 8) + 7],

			});
			// logger.debug("dbls["+i+"] " + dbls[i]);
		}

		return dbls;

	}

	public static final double[] Bytes2ArrayDouble(byte[] data,
			boolean bigendian) throws MSMSException {

		if (data == null)
			return null;

		// ----------

		if (data.length % 8 != 0)
			return null;

		double[] dbls = new double[data.length / 8];

		if (bigendian) {

			for (int i = 0; i < dbls.length; i++) {

				dbls[i] = toDouble(new byte[] {

				data[(i * 8)],

				data[(i * 8) + 1],

				data[(i * 8) + 2],

				data[(i * 8) + 3],

				data[(i * 8) + 4],

				data[(i * 8) + 5],

				data[(i * 8) + 6],

				data[(i * 8) + 7],

				});
				// logger.debug("dbls["+i+"] " + dbls[i]);
			}
			// return toDoubleA(decodedArr);
		} else {
			// throw new
			// ProticPortException("decoding of non big endian bytes not implemented");
			for (int i = 0; i < dbls.length; i++) {

				dbls[i] = toDouble(new byte[] {

				data[(i * 8) + 7],

				data[(i * 8) + 6],

				data[(i * 8) + 5],

				data[(i * 8) + 4],

				data[(i * 8) + 3],

				data[(i * 8) + 2],

				data[(i * 8) + 1], data[(i * 8)]

				});
				// logger.debug("dbls["+i+"] " + dbls[i]);
			}
		}
		return dbls;
		/*
		 * int length = decodedArr.length / 8; double[] result = new
		 * double[length]; byte[] swap = new byte[8]; int k = 0; for (int i = 0;
		 * i < decodedArr.length; i += 8, k++) { logger.debug("i " + i); for
		 * (int j = 0; j < 8; j++) { if (bigendian) { swap[j] = decodedArr[i +
		 * j]; } else { swap[7 - j] = decodedArr[i + j]; } } result[k] =
		 * byteArraytoDouble(swap); } return result;
		 */
	}

	private void setBase64Peaks(String data, boolean isPrecision64,
			boolean bigendian, String pair_order) throws MSMSException {
		// public static final byte[] ArrayDouble2Bytes(double[] arr_double,
		// boolean bigendian) {
		// logger.debug("setBase64Peaks begin is64bits" + isPrecision64 +
		// " isbigendian" + bigendian + " pairorder" + pair_order);
		int length = 0;
		if (isPrecision64 == false) {
			float[] arr_float;
			arr_float = Bytes2ArrayFloat(Base64.decode(data.getBytes()),
					bigendian);
			length = arr_float.length;
			// logger.debug("arr peaks length " + length);
			this.intenArr = new double[length / 2];
			this.mzArr = new double[length / 2];
			int j = 0;
			for (int i = 0; i < length; i += 2, j++) {
				// logger.debug("i " + i + " j " + j + " length " + length);
				if (pair_order.equals("m/z-int")) {
					this.mzArr[j] = arr_float[i];
					this.intenArr[j] = arr_float[i + 1];
				} else {
					this.intenArr[j] = arr_float[i];
					this.mzArr[j] = arr_float[i + 1];
				}
			}
		} else {
			double[] arr_double;
			arr_double = Bytes2ArrayDouble(Base64.decode(data.getBytes()),
					bigendian);
			length = arr_double.length;
			this.intenArr = new double[length];
			this.mzArr = new double[length];
			int j = 0;
			for (int i = 0; i < length; i += 2, j++) {
				if (pair_order.equals("m/z-int")) {
					this.mzArr[j] = arr_double[i];
					this.intenArr[j] = arr_double[i + 1];
				} else {
					this.intenArr[j] = arr_double[i];
					this.mzArr[j] = arr_double[i + 1];
				}
			}
		}
		// for (int i = 0; i < mzArr.length; i++) {
		// logger.debug("mz i=" + i + " value=" + mzArr[i]);
		// }
	}

}
