package fr.inra.pappso.xtandempipeline.sax_parse;

/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;

public class HandlerMzxmlSummary extends SaxBaseHandler {
	private static final Logger logger = Logger
			.getLogger(HandlerMzxmlSummary.class);
	private Boolean isMzXml = null;
	private HashMap<Integer, Long> hashCounterLevel = new HashMap<Integer, Long>();
	private HashMap<Integer, Double> hashIntensitySum = new HashMap<Integer, Double>();
	private Pattern rtPattern;
	private Double rtMin = 100000.0;
	private Double rtMax = 0.0;

	public HandlerMzxmlSummary() throws MSMSException {
		rtPattern = Pattern.compile("^PT(.*)S$");
	}

	@Override
	protected void startUnknownElement(String nameSpaceUri, String sName,
			String qName, Attributes attrs) throws MSMSException,
			SAXNotRecognizedException {
		if (this.isMzXml == null) {
			logger.debug("startUnknownElement " + qName);
			if (qName.equals("mzXML")) {
				this.isMzXml = true;
			} else {
				this.isMzXml = false;
				throw new SAXNotRecognizedException("this is not an mzXML file");
			}
		}
	}

	public void startElementmzXML(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		logger.debug("startElementmzXML begin");
		this.isMzXml = true;
	}

	@Override
	public void endDocument() throws SAXException {
	}

	public void startElementscan(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		// logger.debug("startElementscan begin ");
		/*
		 * <scan polarity="+" highMz="1829.35" totIonCurrent="1.07252e+008"
		 * peaksCount="732" scanType="Full" retentionTime="PT300.03S"
		 * basePeakIntensity="1.08706e+006" msLevel="1" num="1" lowMz="400.631"
		 * basePeakMz="462.039"> <peaks byteOrder="network" pairOrder="m/z-int"
		 * precision="32"></peaks> </scan>
		 */
		Integer msLevel = null;
		if (attrs.getValue("msLevel") != null) {
			msLevel = new Integer(attrs.getValue("msLevel"));

			if (this.hashCounterLevel.containsKey(msLevel)) {
				Long count = this.hashCounterLevel.get(msLevel) + 1;
				this.hashCounterLevel.put(msLevel, count);
			} else {
				this.hashCounterLevel.put(msLevel, new Long(1));
			}
		}
		if (msLevel != null) {
			if (attrs.getValue("totIonCurrent") != null) {
				Double intensity = new Double(attrs.getValue("totIonCurrent")
						.replaceAll("e", "E"));

				if (this.hashIntensitySum.containsKey(msLevel)) {
					Double count = this.hashIntensitySum.get(msLevel)
							+ intensity;
					this.hashIntensitySum.put(msLevel, count);
				} else {
					this.hashIntensitySum.put(msLevel, intensity);
				}
			}

			if (msLevel > 1) {
				if (attrs.getValue("retentionTime") != null) {
					Matcher m = rtPattern.matcher(attrs
							.getValue("retentionTime"));
					if (m.find()) {
						// logger.debug("retentionTime " + m.group(1));
						Double rt = new Double(m.group(1));
						if (rtMin > rt) {
							rtMin = rt;
						}
						if (rtMax < rt) {
							rtMax = rt;
						}
					}
				}
			}
		}
	}

	public HashMap<Integer, Long> getMsLevelCounter() {
		return hashCounterLevel;
	}

	public HashMap<Integer, Double> getMsLevelIntensitySum() {
		return this.hashIntensitySum;
	}

	public Double getRtMin() {
		return this.rtMin;
	}

	public Double getRtMax() {
		return this.rtMax;
	}
}
