package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;

public class HandlerXtResultFile4Spectrum extends SaxBaseHandler {

	private static final Logger logger = Logger
			.getLogger(HandlerXtResultFile4Spectrum.class);

	private Peptide pep_;

	private boolean intensity_value;

	private boolean mz_value;

	private float factor_intensity;

	private boolean foundMz = false;

	private boolean foundIntensities = false;

	private int scanNumToFind = 0;

	private String currentGroupType;

	public HandlerXtResultFile4Spectrum(Peptide pep) throws MSMSException {
		super();
		pep_ = pep;
		logger.info("Load MSMS spectra of : "
				+ pep_.getHashSampleScan().toString());
		this.scanNumToFind = pep_.get_scan();
	}

	@Override
	public void startDocument() throws SAXException {
		// is_spectra = false;
		intensity_value = false;
		mz_value = false;
		// valid_spectra = false;
	}

	@Override
	public void endDocument() throws SAXException {
		if (foundMz == false) {
			try {
				String message = "mz not found for peptide "
						+ this.pep_.getHashSampleScan().toString()
						+ " X!Tandem reference : " + scanNumToFind
						+ ".spectrum";
				logger.warn(message);
			} catch (MSMSException e) {
				throw new SAXException(e);
			}
		}
		if (foundIntensities == false) {
			try {
				String message = "intensities not found for peptide "
						+ this.pep_.getHashSampleScan().toString()
						+ " X!Tandem reference : " + this.scanNumToFind;
				logger.warn(message);
			} catch (MSMSException e) {
				throw new SAXException(e);
			}
		}
	}

	public void startElementgroup(String nameSpaceUri, Attributes attrs) {
		// logger.debug("startElementgroup begin");
		// <group label="performance parameters" type="parameters">
		this.currentGroupType = attrs.getValue("type");
		if (currentGroupType.equals("model")) {
			factor_intensity = Float.valueOf(attrs.getValue("fI"));
		}
	}

	public void endElementgroup(String nameSpaceUri, String data)
			throws MSMSException {
		// <group label="performance parameters" type="parameters">
		this.currentGroupType = null;
	}

	public void startElementGAML__Xdata(String nameSpaceUri, Attributes attrs) {
		// logger.debug("startElementXdata");
		// logger.debug("startElementXdata GAML");
		if (attrs.getValue("label").equals(this.scanNumToFind + ".spectrum")) {
			mz_value = true;
		}

	}

	public void startElementGAML__Ydata(String nameSpaceUri, Attributes attrs) {
		if (attrs.getValue("label").equals(this.scanNumToFind + ".spectrum")) {
			intensity_value = true;
		}

	}

	public void endElementGAML__values(String nameSpaceUri, String data) {
		if (mz_value) {
			pep_.setMz_values(data);
			mz_value = false;
			foundMz = true;
		}
		if (intensity_value) {
			pep_.setIntensity_values(data, factor_intensity);
			intensity_value = false;
			foundIntensities = true;
		}

	}

	public void setScanNumToFind(int scanNumToFind) {
		logger.debug("set scan num to find " + pep_.get_scan() + "=>"
				+ scanNumToFind);

		this.scanNumToFind = scanNumToFind;
	}
}
