package fr.inra.pappso.xtandempipeline.sax_parse;

import java.util.Vector;

import org.apache.log4j.Logger;

import com.compomics.mascotdatfile.util.interfaces.Modification;
import com.compomics.mascotdatfile.util.mascot.MascotDatfile;
import com.compomics.mascotdatfile.util.mascot.PeptideHit;
import com.compomics.mascotdatfile.util.mascot.ProteinHit;
import com.compomics.mascotdatfile.util.mascot.Query;
import com.compomics.mascotdatfile.util.mascot.QueryToPeptideMap;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataFile;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.PeptideStore;

public class Mascot {
	private static final Logger logger = Logger.getLogger(Mascot.class);

	private IdentificationDataFile sample_;
	private Identification identification_;

	private MascotDatfile mascotDatfile;

	// private Peptide peptide;

	private Protein prot_;

	private Match match_;

	private boolean phosphoFilter = false;

	public Mascot(Identification L, IdentificationDataFile sample) {
		// Identification is null if want a new one!
		setIdentification(L);
		this.sample_ = sample;
	}

	public IdentificationDataFile getSample() {
		return sample_;
	}

	public Identification getIdentification() {
		return identification_;
	}

	private void setIdentification(Identification identification_) {
		if (identification_ == null)
			this.identification_ = new Identification();
		else
			this.identification_ = identification_;
	}

	public void readAndParseMascotFile() throws Exception {
		// Load Mascot file
		logger.info("Read :" + sample_.getName());
		mascotDatfile = new MascotDatfile(sample_.getAbsolutePath());

		// Add sample to identification
		identification_.getMsRunSet().getInstance(sample_);

		// Get liste of query and associated peptide
		QueryToPeptideMap map = mascotDatfile.getQueryToPeptideMap();

		for (Object q : mascotDatfile.getQueryList()) {
			Query query = (Query) q;
			Vector<?> peptides = map.getAllPeptideHits(query.getQueryNumber());
			if (peptides != null) {
				// Liste of peptideHit
				for (Object ob : peptides) {
					PeptideHit pep = (PeptideHit) ob;
					addPeptideToIdentification(pep, query, false);
				}
			}
		}

		// Get decoy search
		map = mascotDatfile.getDecoyQueryToPeptideMap();
		if (map != null) {
			for (Object q : mascotDatfile.getQueryList()) {
				Query query = (Query) q;
				Vector<?> peptides = map.getAllPeptideHits(query
						.getQueryNumber());
				if (peptides != null) {
					// Liste of peptideHit
					for (Object ob : peptides) {
						PeptideHit pep = (PeptideHit) ob;
						addPeptideToIdentification(pep, query, true);
					}
				}
			}
		}

	}

	protected void addPeptideToIdentification(PeptideHit pep, Query query,
			boolean reversed) throws MSMSException {

		// mapping proteinHit to protein && peptideHit to peptide
		for (Object obj : pep.getProteinHits()) {
			ProteinHit prot = (ProteinHit) obj;

			// peptide = PeptideStore.getInstance().newPeptide();
			Peptide pep_ = PeptideStore.getInstance().newPeptide();

			pep_.set_sample_id(
					identification_.getMsRunSet().getInstance(sample_), sample_);

			pep_.set_charge((int) round(
					(pep.getPeptideMr() / query.getPrecursorMZ()), 0));
			pep_.set_deltamass((float) pep.getDeltaMass());
			pep_.set_hypercorr((float) pep.getIonsScore());
			pep_.set_mhplus_obser((float) (pep.getPeptideMr()
					+ pep.getDeltaMass() + 1.007825));
			pep_.set_mhplus_theo((float) (pep.getPeptideMr() + 1.007825));
			pep_.set_sequence(pep.getSequence());

			if (XtandemPipelineSession.getInstance().getConfig()
					.get_mascot_evalue_theoritical()) {
				// Evalue theoritical

				pep_.set_evalue((float) pep.getExpectancy());
			} else {
				// Evalue experimental
				double eval = (pep.getIonsScore() - pep.getHomologyThreshold()) / 10;
				eval = 0.05 / Math.pow(10, eval);
				pep_.set_evalue((float) eval);
			}

			// Add first query
			pep_.set_scan(query.getQueryNumber());
			// If scan exist, add it
			try {
				int scan = Integer.parseInt(query.getScans());
				pep_.set_scan(scan);
			} catch (Exception e) {
				logger.debug("No scan for query :" + query.getQueryNumber());
			}
			String rt = query.getRetentionTimeInSeconds();
			if (rt != null)
				pep_.set_RT(rt);

			prot_ = new Protein();
			match_ = new Match();
			// Description including reversed tag for decoy search
			String description = mascotDatfile.getProteinMap()
					.getProteinDescription(prot.getAccession());
			if (reversed)
				description = prot.getAccession() + "|reversed " + description;
			else
				description = prot.getAccession() + " " + description;

			prot_.set_description(description);
			prot_.set_sequence("");
			prot_.setDatabase(identification_.getDatabaseSet().getInstance(
					"Unknown"));

			// Get existing match or create a new one
			if (identification_.contain_protein(prot_)) {
				match_.set_protein_match(prot_);
				identification_.add_protein(prot_, match_);
			}

			else {
				match_ = (Match) identification_.get_match(prot_);
				prot_ = match_.get_protein_match();
			}

			// Add specific peptide param to protein
			pep_.set_post("");
			pep_.set_pre("");
			// modified posi in prot of the peptide
			pep_.set_start(prot.getStart());
			pep_.set_stop(prot.getStop());

			// modifs add with position in prot!!!
			Modification[] mods = pep.getModifications();
			for (int i = 0; i < mods.length; i++) {
				Modification mod = mods[i];
				// N-ter/C-ter correction
				int posi = i;
				if (i > 0) {
					posi--;
				}
				if (i == mods.length - 1) {
					posi--;
				}
				if (mod != null) {
					pep_.set_Modifs(mod.getLocation(), posi + prot.getStart(),
							(float) mod.getMass());
					identification_.add_modifs_to_liste(new Modifs((float) mod
							.getMass()));
				}
			}

			// Add result to identification
			if (phosphoFilter) {
				// Add result on phospho mode
				if (pep_.getGrpPhosphoModifs().size() > 0) {
					identification_.add_peptide(pep_);
					match_.add_peptide_match(pep_);
				}
			} else {
				// add result on normal mode
				identification_.add_peptide(pep_);
				match_.add_peptide_match(pep_);
			}

		}
	}

	public double round(double what, int howmuch) {
		return (double) ((int) (what * Math.pow(10, howmuch) + .5))
				/ Math.pow(10, howmuch);
	}

	public boolean isPhosphoFilter() {
		return phosphoFilter;
	}

	public void setPhosphoFilter(boolean phosphoFilter) {
		this.phosphoFilter = phosphoFilter;
	}
}
