package fr.inra.pappso.xtandempipeline.thread;

/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFile;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.xtandem_param;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemDatabaseXmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemParamXmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemResultFile;
import fr.inra.pappso.xtandempipeline.output.MsmsTextOutput;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerHTCondorQueue;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem_params;

public class XtandemCondorAnalysis extends xtandem_analysis {
	private static final Logger logger = Logger
			.getLogger(XtandemCondorAnalysis.class);
	private Integer condorJobNumber = null;
	private Integer condorClusterNumber = null;
	private File condor_q;
	private String condorQstatus;
	private int completedJobs = -1;
	private int totalJobs;

	public XtandemCondorAnalysis(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Xtandem Condor analysis start");

		if (conf.getCondorSubmitCommand() == null) {
			String message = "unable to submit HTCondor jobs\n \"condor_submit\" not found";
			logger.error(message);
			throw new MSMSException(message);
		}
		condor_q = conf.getCondorQueueCommand();
		if (condor_q == null) {
			String message = "unable to survey HTCondor jobs\n \"condor_q\" not found";
			logger.error(message);
			throw new MSMSException(message);
		}
		// creation du fichier database
		// File infile = new File(this.save_dir.getAbsolutePath()
		// + System.getProperty("file.separator") + "infile.xml");

		// Correction du fichier de parametres
		logger.info("Corrected xtandem preset error");
		xtandem_params params = new xtandem_params();
		params.load_xml(model);
		// params.save_to_xml(model);

		this.set_view("Submitting job(s) to HTCondor\n" + "Please wait");
		File condorSubmitFile = newCondorSubmitFile(params);

		Process pr = null;
		pr = Runtime.getRuntime().exec(
				conf.getCondorSubmitCommand().getAbsolutePath() + " "
						+ condorSubmitFile.getAbsolutePath());

		OutputStream out = pr.getOutputStream();
		out.write("input text".getBytes());
		out.close();
		BufferedReader in = new BufferedReader(new InputStreamReader(
				pr.getInputStream()));
		String line;
		pr.waitFor();
		while ((line = in.readLine()) != null) {
			if (this.condorClusterNumber == null) {
				parseCondorJobNumber(line);
			}
		}
		in.close();

		this.set_view("HTCondor job(s) submitted\n" + this.condorJobNumber
				+ " job(s) submitted to cluster " + this.condorClusterNumber
				+ ".");
		logger.info("HTCondor job submitted " + condorClusterNumber);
		while (condorIsRunning()) {
			this.set_view(condorQstatus);
			Thread.currentThread();
			Thread.sleep(2000);
			if (this.isStop()) {
				// pipe.stop_process();
				stopCondor();
			}
			this.add_current_progress();
		}
		// this.add_current_progress();

	}

	private void stopCondor() throws MSMSException, StopException {
		throw new StopException(
				"Unable to stop jobs running in HTCondor, please use :\n\"condor_rm "
						+ this.condorClusterNumber + "\"");
	}

	private boolean condorIsRunning() throws MSMSException {
		boolean isRunning = true;
		try {
			Process pr = null;
			pr = Runtime.getRuntime().exec(
					condor_q.getAbsolutePath() + " " + this.condorClusterNumber
							+ " -xml");

			OutputStream out = pr.getOutputStream();
			out.write("input text".getBytes());
			out.close();

			HandlerHTCondorQueue condorqHandler = new HandlerHTCondorQueue(this);

			pr.waitFor();
			new parse_file(condorqHandler, pr.getInputStream());

			if (this.completedJobs == this.condorJobNumber) {
				isRunning = false;
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return isRunning;
	}

	private void parseCondorJobNumber(String condor_job) throws MSMSException {
		// Submitting job(s)...
		// 3 job(s) submitted to cluster 3.
		Pattern lastchancepattern = Pattern
				.compile("([0-9]*) job\\(s\\) submitted to cluster ([0-9]*).");
		Matcher m = lastchancepattern.matcher(condor_job);
		// logger.debug("avant macth findMsSampleByMicroplate "+orderNum+"_"+plateNum+"_"+wellNum);
		// System.out.println("avant macth findMsSampleByMicroplate "+orderNum+"_"+plateNum+"_"+wellNum);
		logger.debug(condor_job);
		if (m.find()) {
			logger.debug(m.group(1) + " " + m.group(2));
			condorClusterNumber = new Integer(m.group(2));
			condorJobNumber = new Integer(m.group(1));

		} else {
			// String message = "unable to find HTCondor job numbers in "
			// + condor_job;
			// logger.error(message);
			// throw new MSMSException(message);
		}
	}

	private File newCondorSubmitFile(xtandem_params params)
			throws MSMSException {
		xtandem_param paramThreads = params.get_param_to("spectrum, threads");
		int requestedCpus = 0;
		if (paramThreads != null) {
			// get requested number of cpus from model :
			requestedCpus = Integer.valueOf(paramThreads.getValue());

		}

		createTempDirectoryForCondor();
		XtandemDatabaseXmlFile databasexml = new XtandemDatabaseXmlFile(tmpDir,
				databases);
		File condorSubmitTxtFile;

		condorSubmitTxtFile = new File(tmpDir.getAbsolutePath()
				+ System.getProperty("file.separator") + "submit_condor.txt");
		// File.createTempFile("xtandem_condor", ".txt");
		File condorTandemLogFile = new File(tmpDir.getAbsolutePath()
				+ System.getProperty("file.separator") + "submit_condor.log");

		// copy model in tmpdir :
		XtandemModelFile modelTmp = new XtandemModelFile(new File(
				tmpDir.getAbsolutePath() + System.getProperty("file.separator")
						+ model.getName()));
		try {
			// xtandem_params params = new xtandem_params();
			params.load_xml(model);
			// params.get_param_to("spectrum, threads").setValue("1");
			params.save_to_xml(modelTmp);
		} catch (Exception e) {
			throw new MSMSException("Unable to copy model\n" + e.getMessage());
		}

		/*
		 * copy database.xml xtandem_param3542072178348339854.xml
		 * xtandem_param509267883628656385.xml submit_condor.log
		 * xtandem_param3587450986871313885.xml
		 * xtandem_param8058920024180583494.xml submit_condor.txt
		 */
		MsmsTextOutput out = new MsmsTextOutput();
		out.open(condorSubmitTxtFile.getAbsolutePath());
		out.write("Universe   = vanilla" + "\n");
		out.write("Rank       = Mips" + "\n");
		// out.write("request_cpus = " + requestedCpus + "\n");
		if ((requestedCpus > 0) && (requestedCpus < 200)) {
			out.write("request_cpus = " + requestedCpus + "\n");
		}

		out.write("Executable = " + conf.getXtandem_link().getAbsolutePath()
				+ "\n");
		out.write("Log        = " + condorTandemLogFile.getAbsolutePath()
				+ "\n");
		out.write("Output        = " + tmpDir.getAbsolutePath()
				+ System.getProperty("file.separator")
				+ "tandem.$(Process).out" + "\n");
		out.write("Error        = " + tmpDir.getAbsolutePath()
				+ System.getProperty("file.separator")
				+ "tandem.$(Process).error" + "\n");
		for (PeakListFile pkFile : this.peakListFiles) {
			logger.info("Xtandem analysis on : " + pkFile.getName());
			out.write("\n");
			XtandemResultFile result_out = new XtandemResultFile(
					this.getSave_dir(), pkFile);
			XtandemParamXmlFile infile = new XtandemParamXmlFile(this.tmpDir,
					modelTmp, databasexml, pkFile, result_out);
			out.write("Arguments = " + infile.getAbsolutePath() + "\n");
			out.write("Queue" + "\n");
			out.write("\n");
		}
		// transfer_input_files = < file1,file2,file... >
		out.close();

		logger.info("condor submit job file written in : "
				+ condorSubmitTxtFile.getAbsolutePath());
		return condorSubmitTxtFile;
	}

	public File createTempDirectoryForCondor() throws MSMSException {
		try {
			tmpDir = File.createTempFile("temp_condor_job",
					Long.toString(System.nanoTime()));
			if (!(tmpDir.delete())) {
				throw new IOException("Could not delete temp file: "
						+ tmpDir.getAbsolutePath());
			}
			if (conf.getCondorSharedTmpDir() == null) {
			} else {
				tmpDir = new File(conf.getCondorSharedTmpDir()
						+ System.getProperty("file.separator")
						+ tmpDir.getName());
			}
		} catch (IOException e) {
			String message = "unable to create temporary directory for condor logs and outputs";
			logger.error(message);
			throw new MSMSException(message);
		}

		if (!(tmpDir.mkdir())) {
			String message = "Could not create temp directory: "
					+ tmpDir.getAbsolutePath();
			logger.error(message);
			throw new MSMSException(message);
		}
		if (!(tmpDir.setWritable(true, false))) {
			String message = "Could not set write permission to everybody: "
					+ tmpDir.getAbsolutePath();
			logger.error(message);
			throw new MSMSException(message);
		}
		logger.info("tmp dir for condor is " + tmpDir.getAbsolutePath());
		return (tmpDir);
	}

	public void setStatusMessage(String statusMessage) {
		this.condorQstatus = statusMessage;
	}

	public void setJobCompleted(int junexpanded, int jidle, int jrunning,
			int jremoved, int jcompleted, int jheld, int jsubmission_error) {
		this.completedJobs = jcompleted;
		this.totalJobs = junexpanded + jidle + jrunning + jremoved + jcompleted
				+ jheld + jsubmission_error;
	}

	public int getCondorJobNumber() {
		return this.condorJobNumber;
	}

}
