package fr.inra.pappso.xtandempipeline.thread;

import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.filter_print.filter_index_and_order_match_scan;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_evalue;

public class individual extends XtpProcess {
	protected static final Logger logger = Logger.getLogger(individual.class);

	private Vector<IdentificationDataSource> dataSources;

	// private ArrayList<Identification> identifications =
	// XtandemPipelineSession
	// .getInstance().getIdentificationList();

	public individual(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Process result in Individual mode");

		ArrayList<Identification> identifications = XtandemPipelineSession
				.getInstance().getIdentificationList();

		// Ecrire le process et penser à relever des StopException
		for (int i = 0; i < dataSources.size(); i++) {
			IdentificationDataSource dataSourceToParse = dataSources.get(i);
			// parsing
			this.set_view("Analysing file : " + (i + 1) + "/"
					+ dataSources.size() + "\n" + "Parsing file : "
					+ dataSourceToParse.getName());

			// Is sample exist? Get previous identification
			Identification identtemp = null;
			for (int m = 0; m < identifications.size(); m++) {
				logger.debug("identifications.size() " + identifications.size());
				MsRunSet samples = identifications.get(m).getMsRunSet();
				for (MsRun sample : samples) {
					logger.debug("msRun name : " + sample.getSampleName());
				}
				if (samples.containsMsRunFromThisDataSource(dataSourceToParse)) {
					identtemp = identifications.get(m);
					logger.debug("found " + dataSourceToParse.getName());
				}
			}
			// xtandem handler;
			// if (identtemp != null)
			// handler = new xtandem(identtemp, dataSources.get(i));
			// else
			// handler = new xtandem(dataSources.get(i));
			// new parse_file(handler, dataSources.get(i).);
			//
			// if (identtemp == null)
			// identifications.add(handler.get_identifications());

			// Load identification from datasource
			identifications
					.add(dataSourceToParse.loadIdentification(identtemp));

			// verification pas arreté
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();
		}
		// identification ident = handler.get_identifications();

		for (int j = 0; j < identifications.size(); j++) {
			Identification ident = identifications.get(j);
			// filtering
			this.set_view("Analysing file : " + (j + 1) + "/"
					+ identifications.size() + "\n" + "Filtering file : "
					+ ident.getMsRunSet().getMsRunList().get(0).getSampleName());

			// on mets à jour les liens
			ident.filtered_match(new filter_index_and_order_match_scan());

			ident.filtered_match(new filter_peptide_evalue(conf
					.get_peptide_evalue(), conf
					.getMinimumPeptideProphetProbability(), conf
					.getMinimumPeptideInterProphetProbability()));
			// modification de la evalue sample
			ident.filtered_match(new filter_recalcul_protein_evalue());

			ident.filtered_match(new filter_protein_evalue_to(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number_to(conf
					.get_peptide_number()));
			// }

			// creation des submatchs
			ident.performedGroupSet();

			// elimination des protéines contaminantes
			ident.remove_contaminant_submatch(conf.get_database_filter());

			// Verification pas arrete
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();
		}

	}

	@Override
	public void setDataSource(Vector<IdentificationDataSource> dataSource) {
		this.dataSources = dataSource;
		this.set_max_progress(dataSources.size() * 2);
	}

}
