package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;

import org.apache.log4j.Logger;

import fr.inra.pappso.csv2ods.TsvWriterOneSheet;
import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoComparSpectraOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoPeptideOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoProteinOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoSpectraOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPhosphoStatsOutput;

public class printing_phosphopeptide extends printing_base {
	protected static final Logger logger = Logger
			.getLogger(printing_phosphopeptide.class);

	public printing_phosphopeptide(config conf) {
		super(conf);
	}

	@Override
	protected void spreadSheetReport() throws Exception {
		logger.info("spreadSheetReport begin");

		this.set_view("Printing protein file results");

		OdsDocWriterInterface odsDoc = null;
		try {
			if (this.isCsv) {

			} else {
				odsDoc = new OdsDocWriter(this.getfileout().getAbsolutePath()
						+ ".ods");
			}

			logger.debug("Print new proteins");
			this.printPhosphoProteinList(odsDoc);
			logger.debug("Print new peptides");
			this.printPhosphoPeptideList(odsDoc);
			logger.debug("Print new compar spectra");
			this.printPhosphoComparSpectraList(odsDoc);
			if (this.isExpertMode) {
				logger.debug("Print new spectra list");
				this.printPhosphoSpectraList(odsDoc);
				logger.debug("Print new phospho stats list");
				this.printPhosphoStatsList(odsDoc);
			}

			logger.debug("Print XTP running informations");
			this.printXtpRunningDetails(odsDoc);
			// odsDoc.close();
		} catch (Exception e) {
			throw e;
		} finally {
			if (odsDoc != null) {
				odsDoc.close();
			}
		}
		this.add_current_progress();
	}

	private void printPhosphoSpectraList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPhosphoSpectraList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing spectra results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_spectra.txt"));
		} else {
		}

		OdsPhosphoSpectraOutput writer = new OdsPhosphoSpectraOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoSpectraList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_spectra.txt");
		}

	}

	private void printPhosphoComparSpectraList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPhosphoComparSpectraList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing compar spectra results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_compar_spectra.txt"));
		} else {
		}

		OdsPhosphoComparSpectraOutput writer = new OdsPhosphoComparSpectraOutput(
				odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoComparSpectraList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath()
					+ "_compar_spectra.txt");
		}
	}

	private void printPhosphoPeptideList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPhosphoPeptideList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing peptide list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_peptides.txt"));
		} else {
		}

		OdsPhosphoPeptideOutput writer = new OdsPhosphoPeptideOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoPeptideList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_peptides.txt");
		}
	}

	private void printPhosphoProteinList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPhosphoProteinList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing protein list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_proteins.txt"));
		} else {
		}

		OdsPhosphoProteinOutput writer = new OdsPhosphoProteinOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPhosphoProteinList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_proteins.txt");
		}
	}

	private void printPhosphoStatsList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPhosphoStatsList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing phoso stats results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_phospho_stats.txt"));
		} else {
		}

		OdsPhosphoStatsOutput writer = new OdsPhosphoStatsOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			// this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
			writer.writeStats();
		}
		logger.debug("printPhosphoStatsList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath()
					+ "_phospho_stats.txt");
		}

	}
}
