package fr.inra.pappso.xtandempipeline.thread;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_redondancy;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_evalue;

public class reinitialise_result_filter_and_create_grouping extends XtpProcess {
	protected static final Logger logger = Logger
			.getLogger(reinitialise_result_filter_and_create_grouping.class);

	private Identification ident;

	private DataType_Base data_type;

	public reinitialise_result_filter_and_create_grouping(config conf,
			Identification ident, DataType_Base data_type) {
		super(conf);
		this.ident = ident;
		this.data_type = data_type;
		this.set_max_progress(6);
	}

	protected void processing() throws Exception {
		logger.info("Reinitialise result and performed filter and grouping");

		// reinitilaise les peptides
		this.set_view("Reinitialise peptide");
		for (Peptide pep : ident.get_peptide_liste())
			pep.setIsValid(true);
		this.add_current_progress();

		// reinitialise les proteins
		this.set_view("Reinitialise proteins");
		for (Match m : ident.get_protein_liste().values()) {
			m.setIsvalidate(true);
		}
		this.add_current_progress();

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");

		this.set_view("Filtering peptides");
		// recalcul de la evalue proteique
		ident.filtered_match(new filter_peptide_evalue(conf
				.get_peptide_evalue(), conf
				.getMinimumPeptideProphetProbability(), conf
				.getMinimumPeptideInterProphetProbability()));
		if (data_type.getIs_phosphopeptide()) {
			ident.filtered_match(new filter_peptide_redondancy());
		}

		this.set_view("Filtering proteins");
		// recalcul de la evalue proteique
		ident.filtered_match(new filter_recalcul_protein_evalue());

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_protein_filter_to_complete()) {
			ident.filtered_match(new filter_protein_evalue(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number(conf
					.get_peptide_number()));
		} else {
			ident.filtered_match(new filter_protein_evalue_to(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number_to(conf
					.get_peptide_number()));
		}
		this.add_current_progress();

		// creation des submatchs
		this.set_view("Grouping \n"
				+ "Be patient, this step can take some time");
		if (data_type.is_phosphopeptide) {
			ident.performedPhosphoGroupSet();
		} else {
			ident.performedGroupSet();
		}

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// no renleve les protéines contaminantes
		this.set_view("Remove contaminants");
		ident.remove_contaminant_submatch(conf.get_database_filter());

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();
	}

	public void setData_type(DataType_Base data_type) {
		this.data_type = data_type;
	}

	public DataType_Base getData_type() {
		return data_type;
	}

	public Identification getIdent() {
		return ident;
	}

	public void setIdent(Identification ident) {
		this.ident = ident;
	}

}
