package fr.inra.pappso.xtandempipeline.ui.swt;

import java.io.File;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFile;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFileSet;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFile;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFileSet;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;
import fr.inra.pappso.xtandempipeline.thread.xtandem_analysis;

public class Xtandem_Analysis_Param {
	private static final Logger logger = Logger
			.getLogger(Xtandem_Analysis_Param.class);

	protected config conf;
	private boolean Cancelled;
	protected Shell shell;
	private PeakListFileSet add_peak_list;
	private String current_directory;
	private xtandem_analysis xtandemAnalysis;
	private List databaseSwtList;
	private Button next;
	protected Combo comboPresetListModels;
	private File out_dir;
	private Button bEditPreset;

	public Xtandem_Analysis_Param(Main_Windows controlleur, config X,
			xtandem_analysis xtandem) {

		logger.debug("Xtandem_Analysis_Param begin");
		this.conf = X;
		this.xtandemAnalysis = xtandem;
		this.Cancelled = true;
		current_directory = conf.getRaw_data().getAbsolutePath();
		out_dir = conf.getXmlresult();
		add_peak_list = new PeakListFileSet();
		// initialise la fenetre
		shell = new Shell(controlleur.getDisplay(), SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);
		shell.setImages(controlleur.getImages());
		init();

		shell.pack();
		// shell.setSize(500, 600);
		load_data_fasta();
		load_data_model(false);
		shell.open();
		while (!shell.isDisposed()) {
			if (!controlleur.getDisplay().readAndDispatch()) {
				controlleur.getDisplay().sleep();
			}
		}
	}

	private void load_data_fasta() {
		logger.debug("load_data_fasta begin");
		FastaFileSet fastaFiles = conf.getprotein_databases();
		databaseSwtList.removeAll();
		for (FastaFile f : fastaFiles)
			databaseSwtList.add(f.getName());
		databaseSwtList.setData("path", conf.getProtein_database().getAbsolutePath());
	}

	private void load_data_model(Boolean save_current_posi) {
		logger.debug("load_data_model begin");
		ArrayList<XtandemModelFile> models = conf.getXtandem_models();
		int select = comboPresetListModels.getSelectionIndex();
		comboPresetListModels.removeAll();
		for (XtandemModelFile f : models) {
			comboPresetListModels.add(f.getName());
		}
		comboPresetListModels.setData("path", conf.getXtandemModelsDirectory()
				.getAbsolutePath());
		if (save_current_posi)
			comboPresetListModels.select(select);
		else
			comboPresetListModels.select(0);

		this.bEditPreset.setEnabled(true);
		if (models.size() == 0) {
			this.bEditPreset.setEnabled(false);
		}
	}

	private void init() {
		logger.debug("init begin");
		shell.setText("Parameters for X!Tandem analysis");
		GridLayout layout = new GridLayout(1, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		shell.setLayout(layout);

		Group add_xtandem = new Group(shell, SWT.SHADOW_IN);
		add_xtandem.setText("Set peaklist files to analyze");
		add_xtandem.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		add_xtandem.setLayout(new GridLayout(2, false));

		Text add_files = new Text(add_xtandem, SWT.SINGLE | SWT.READ_ONLY);
		add_files.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		add_files.setText("No selected");
		Button add = new Button(add_xtandem, SWT.PUSH);
		add.setText("Add files");
		add.setData("text", add_files);
		add.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Xtandem_Analysis_Param.this.add_clic((Button) e.widget);
			}
		});

		Group fasta = new Group(shell, SWT.SHADOW_IN);
		fasta.setText("Select protein databases to analyze");
		fasta.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fasta.setLayout(new GridLayout(2, false));

		databaseSwtList = new List(fasta, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER
				| SWT.READ_ONLY);
		databaseSwtList.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		for (int i = 0; i < 3; i++)
			databaseSwtList.add("01234567890124567890123456789012456789012456789");
		databaseSwtList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Xtandem_Analysis_Param.this.verify_all_params();
			}
		});
		Button other = new Button(fasta, SWT.PUSH);
		other.setText("Other");
		other.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Xtandem_Analysis_Param.this.set_fasta_files_by_other();
			}
		});

		Group preset = new Group(shell, SWT.SHADOW_IN);
		preset.setText("Set X!Tandem presets");
		preset.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		preset.setLayout(new GridLayout(3, false));

		this.comboPresetListModels = new Combo(preset, SWT.READ_ONLY
				| SWT.DROP_DOWN);
		this.comboPresetListModels.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));
		bEditPreset = new Button(preset, SWT.PUSH);
		bEditPreset.setText("Edit");
		bEditPreset.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					String model = comboPresetListModels
							.getItem(comboPresetListModels.getSelectionIndex());
					// Vector<File> all_models = (Vector<File>)
					// params.getData();
					new Xtandem_preset(shell, conf, conf
							.getXtandemModelFile(model));
					Xtandem_Analysis_Param.this.load_data_model(true);
				} catch (MSMSException e1) {
					String message = "ERROR : " + e1.getMessage();
					logger.error(message);
				}
			}
		});
		Button new_but = new Button(preset, SWT.PUSH);
		new_but.setText("New");
		new_but.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					new Xtandem_preset(shell, conf, null);
					Xtandem_Analysis_Param.this.load_data_model(false);
				} catch (MSMSException e1) {
					String message = "ERROR : " + e1.getMessage();
					logger.error(message);
				}
			}
		});

		Group save_out = new Group(shell, SWT.SHADOW_IN);
		save_out.setText("Save X!Tandem result in");
		save_out.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		save_out.setLayout(new GridLayout(2, false));

		Text out_dir_text = new Text(save_out, SWT.SINGLE | SWT.READ_ONLY);
		out_dir_text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if (out_dir.exists())
			out_dir_text.setText(out_dir.getAbsolutePath());
		else
			out_dir_text.setText("Not selected");
		Button set = new Button(save_out, SWT.PUSH);
		set.setText("Set");
		set.setData("text", out_dir_text);
		set.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Xtandem_Analysis_Param.this
						.set_directory_to_save((Button) e.widget);
			}
		});

		Label separator = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		separator.setLayoutData(data);

		this.next = new Button(shell, SWT.PUSH);
		next.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		next.setText("Launch X!Tandem");
		next.setEnabled(false);
		next.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Xtandem_Analysis_Param.this.next_clic();
			}
		});
	}

	protected void verify_all_params() {
		logger.debug("verify_all_params begin");
		boolean valid = true;
		String message = "";
		if (this.databaseSwtList.getSelectionCount() == 0) {
			valid = false;
			message += "no database file selected\n";
		}
		if (this.add_peak_list.size() == 0) {
			valid = false;
			message += "no peak list file selected\n";
		}
		if (!this.out_dir.exists() || !this.out_dir.isDirectory()) {
			valid = false;
			message += "output dir does not exists or is not a directory ("
					+ this.out_dir.getAbsolutePath() + ")\n";
		}
		if (!valid) {
			logger.warn(message);
		}
		this.next.setEnabled(valid);

		for (int i : this.databaseSwtList.getSelectionIndices()) {
			logger.debug(i + ":" + this.databaseSwtList.getItem(i));
		}
	}

	private void set_directory_to_save(Button widget) {
		logger.debug("set_directory_to_save begin");
		DirectoryDialog dlg2 = new DirectoryDialog(shell, SWT.SAVE | SWT.SINGLE);
		dlg2.setText("Save the results in folder");
		dlg2.setFilterPath(conf.getXmlresult().getAbsolutePath());
		String S_out = dlg2.open();
		if (S_out != null) {
			out_dir = new File(S_out);
			((Text) widget.getData("text")).setText(out_dir.getAbsolutePath());
		}
		this.verify_all_params();
	}

	private void set_fasta_files_by_other() {
		logger.debug("set_fasta_files_by_other begin");
		FileDialog dlg = new FileDialog(shell, SWT.OPEN | SWT.MULTI);
		dlg.setText("Select protein databases to analyze with X!tandem");
		dlg.setFilterExtensions(new String[] { "*.fasta", "*.*" });
		dlg.setFilterNames(new String[] { "Protein databases", "All files" });
		dlg.setFilterPath(conf.getProtein_database().getAbsolutePath());
		String fileName = dlg.open();
		if (fileName != null) {
			String[] fasta = dlg.getFileNames();
			// Vector<File> fastafile = new Vector<File>();
			// for (int i = 0; i < fasta.length; i++) {
			// fastafile.add(new File(dlg.getFilterPath()
			// + System.getProperty("file.separator") + fasta[i]));
			// }
			databaseSwtList.removeAll();
			int[] select = new int[fasta.length];
			for (int j = 0; j < fasta.length; j++) {
				databaseSwtList.add(fasta[j]);
				select[j] = j;
			}
			databaseSwtList.select(select);
			databaseSwtList.setData("path", dlg.getFilterPath());
		}
		this.verify_all_params();
	}

	private void add_clic(Button but) {
		logger.debug("add_clic begin");
		try {
			add_peak_list.addAll(this.get_file_peaklist());
			Text t = (Text) but.getData("text");
			t.setText(add_peak_list.size() + " files selected");
			t.update();

			this.verify_all_params();
		} catch (StopException S) {

		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
			messageBox.setMessage("Problems during xml files selection :\n"
					+ error);
			messageBox.open();
			e.printStackTrace();
		}
	}

	private PeakListFileSet get_file_peaklist() throws Exception {
		logger.debug("get_file_peaklist begin");
		FileDialog dlg = new FileDialog(shell, SWT.OPEN | SWT.MULTI);
		dlg.setFilterExtensions(new String[] { "*.mzXML", "*.mzData", "*.mzML",
				"*.mgf", "*.pkl", "*.xml", "*.*" });
		dlg.setFilterNames(new String[] { "mzXML files", "mzData files",
				"mzML files", "mgf files", "pkl files",
				"X!Tandem result files", "All files" });
		dlg.setFilterPath(current_directory);
		dlg.setText("Add new peaklist files to analyze");
		String fileName = dlg.open();
		if (fileName == null)
			throw new StopException("No xml file selected");
		String[] f = dlg.getFileNames();
		PeakListFileSet files = new PeakListFileSet();
		for (int i = 0; i < f.length; i++) {
			files.add((new PeakListFile(dlg.getFilterPath()
					+ System.getProperty("file.separator") + f[i])));
		}
		current_directory = dlg.getFilterPath();
		return (files);
	}

	protected void next_clic() {
		logger.debug("next_clic begin");
		try {
			logger.debug("this.xtandemAnalysis.setPeakListFileSet");
			this.xtandemAnalysis.setPeakListFileSet(add_peak_list);

			logger.debug("FastaFileSet");
			FastaFileSet fastaFiles = new FastaFileSet();
			for (int i : this.databaseSwtList.getSelectionIndices()) {
				fastaFiles.add(new FastaFile(this.databaseSwtList.getData("path")
						+ System.getProperty("file.separator")
						+ this.databaseSwtList.getItem(i)));
			}
			logger.debug("setDatabases");
			this.xtandemAnalysis.setDatabases(fastaFiles);

			logger.debug("setModel");
			// Vector<File> all_models = (Vector<File>) this.params.getData();
			this.xtandemAnalysis.setModel(conf
					.getXtandemModelFile(this.comboPresetListModels
							.getItem(this.comboPresetListModels
									.getSelectionIndex())));
			logger.debug("setSave_dir");
			this.xtandemAnalysis.setSave_dir(out_dir);
			logger.debug("next_clic 1");
			// on indique que la fenetre n'a pas ete cancel
			this.Cancelled = false;
			// on libere
			shell.dispose();
			logger.debug("next_clic end");

		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
			messageBox
					.setMessage("Problems trying to start X!Tandem analysis :\n"
							+ error);
			messageBox.open();
			e.printStackTrace();
		}
	}

	public boolean getCancelled() {
		logger.debug("getCancelled begin");
		return Cancelled;
	}
}
