package fr.inra.pappso.xtandempipeline.ui.swt;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.xtandem_param;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem_params;

public class Xtandem_preset {
	private static final Logger logger = Logger.getLogger(Xtandem_preset.class);
	protected config conf;

	protected Shell shell;

	protected Browser help_param;

	private TabFolder tabFolder;

	private xtandem_params params;

	private Text modelname;

	private Combo model_list;

	private Vector<Text> text_field;

	private Vector<Combo> combo_field;

	private boolean is_full;

	public Xtandem_preset(Main_Windows controlleur, config conf)
			throws MSMSException {
		is_full = true;
		logger.debug("edit param xtandem");
		this.conf = conf;
		shell = new Shell(controlleur.getDisplay(), SWT.SHELL_TRIM);
		shell.setImages(controlleur.getImages());
		this.defined_interface();
		// on rempli les données avec un nouveau model
		this.modelname.setText("New model");
		this.set_model_list(null);

		shell.pack();
		shell.open();
	}

	public Xtandem_preset(Shell modal, config conf, XtandemModelFile xmlfilename)
			throws MSMSException {
		is_full = false;
		logger.debug("edit param xtandem");
		this.conf = conf;
		shell = new Shell(modal, SWT.SHELL_TRIM | SWT.APPLICATION_MODAL);
		shell.setImages(modal.getImages());
		this.defined_interface();
		// on rempli les données avec un nouveau model
		this.modelname.setText("New model");
		this.set_model_list(null);

		shell.pack();
		shell.open();

		if (xmlfilename == null) {
			this.restore_default_value();
		} else {
			this.set_model_list(xmlfilename);
			this.load_value_to_xml();
		}

		while (!shell.isDisposed()) {
			if (!modal.getDisplay().readAndDispatch()) {
				modal.getDisplay().sleep();
			}
		}
	}

	private void defined_interface() throws MSMSException {
		try {
			shell.setText("Edit X!Tandem identification parameters");
			shell.setLayout(new GridLayout(1, false));
			Composite save_and_select = new Composite(shell, SWT.NONE);
			save_and_select
					.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			this.set_save_and_select(save_and_select);
			SashForm horizontale = new SashForm(shell, SWT.HORIZONTAL);
			horizontale.setLayoutData(new GridData(GridData.FILL_BOTH));
			this.tabFolder = new TabFolder(horizontale, SWT.TOP);
			this.set_tab_folder();

			this.help_param = new Browser(horizontale, SWT.NONE);
			this.help_param.setText("<p>Help window</p>");

			horizontale.setWeights(new int[] { 6, 4 });
			horizontale.pack();
		} catch (Exception e) {
			String message = "ERROR :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void set_save_and_select(Composite save_and_select)
			throws MSMSException {
		try {
			save_and_select.setLayout(new GridLayout(2, true));

			Group save = new Group(save_and_select, SWT.NONE);
			save.setLayoutData(new GridData(GridData.FILL_BOTH));
			save.setText("Save X!Tandem models as");
			save.setLayout(new GridLayout(3, false));

			Button new_mod = new Button(save, SWT.PUSH);
			new_mod.setText("New");
			new_mod.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					Xtandem_preset.this.restore_default_value();
				}
			});
			new_mod.setEnabled(is_full);
			this.modelname = new Text(save, SWT.SINGLE);
			modelname.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			Button save_mod = new Button(save, SWT.PUSH);
			save_mod.setText("Save");
			save_mod.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					Xtandem_preset.this.save_value_to_xml();
					Xtandem_preset.this.close_on_non_full();
				}
			});

			Group load = new Group(save_and_select, SWT.NONE);
			load.setLayoutData(new GridData(GridData.FILL_BOTH));
			load.setText("Load X!Tandem saved models");
			load.setLayout(new GridLayout(3, false));
			Button delete_mod = new Button(load, SWT.PUSH);
			delete_mod.setText("Delete");
			delete_mod.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					try {
						Xtandem_preset.this.delete_xml_file();
					} catch (MSMSException e1) {
						String message = "ERROR : " + e1.getMessage();
						logger.error(message);
					}
				}
			});
			delete_mod.setEnabled(is_full);
			this.model_list = new Combo(load, SWT.DROP_DOWN | SWT.SINGLE);
			model_list.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			model_list.setEnabled(is_full);
			Button open = new Button(load, SWT.PUSH);
			open.setText("Open");
			open.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					Xtandem_preset.this.load_value_to_xml();
				}
			});
			open.setEnabled(is_full);

			save_and_select.pack();
		} catch (Exception e) {
			String message = "ERROR :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	protected void close_on_non_full() {
		if (!is_full)
			shell.dispose();
	}

	private void set_model_list(XtandemModelFile xmlfilename) {
		logger.debug("set_model_list begin");
		try {
			String name;
			if (xmlfilename == null) {
				name = "New model";
			} else {
				name = xmlfilename.getName();
			}
			ArrayList<XtandemModelFile> xmls = conf.getXtandem_models();
			model_list.removeAll();
			int contain_name = -1;
			for (int i = 0; i < xmls.size(); i++) {
				model_list.add(xmls.get(i).getName());
				if (name.equalsIgnoreCase(xmls.get(i).getName()))
					contain_name = i;
			}
			logger.debug("model_list.getItemCount()="
					+ model_list.getItemCount());
			if (contain_name < 0)
				model_list.select(0);
			else
				model_list.select(contain_name);
		} catch (Exception e) {
			// juste erreur si pas de model
			logger.debug(e.getMessage());
		}
	}

	private void set_tab_folder() {
		try {
			params = new xtandem_params();
			text_field = new Vector<Text>();
			combo_field = new Vector<Combo>();

			for (String groupname : params.get_groups()) {
				TabItem item = new TabItem(tabFolder, SWT.NONE);
				item.setText(groupname);
				Composite grid = new Composite(tabFolder, SWT.NONE);
				GridLayout layout = new GridLayout(3, false);
				layout.verticalSpacing = 5;
				layout.horizontalSpacing = 5;
				grid.setLayout(layout);
				for (xtandem_param param : params
						.get_params_to_group(groupname)) {
					Label l = new Label(grid, SWT.SINGLE | SWT.RIGHT);
					l.setText(param.toString());
					l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
							false));
					if (param.get_list_item().length == 1
							|| !param.isModified()) {

						Text text = new Text(grid, SWT.SINGLE);
						text.setLayoutData(new GridData(
								GridData.FILL_HORIZONTAL));
						text.setData("param", param);
						if (!param.isModified()) {
							text.setEditable(false);
							text.setEnabled(false);
						}
						text.setText(param.getValue());
						text_field.add(text);
					} else {
						Combo com = new Combo(grid, SWT.DROP_DOWN
								| SWT.READ_ONLY);
						for (String value : param.get_list_item())
							com.add(value);
						com.select(param.getValueindex());
						com.setData("param", param);
						com.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
						this.combo_field.add(com);
					}
					Button help = new Button(grid, SWT.PUSH);
					help.setText("?");
					help.setData("link", param.getLink());
					help.addSelectionListener(new SelectionAdapter() {
						public void widgetSelected(SelectionEvent e) {
							InputStreamReader reader = new InputStreamReader(
									this.getClass().getResourceAsStream(
											(String) e.widget.getData("link")));
							BufferedReader br = new BufferedReader(reader);
							String str;
							StringBuffer sb = new StringBuffer();
							try {
								while ((str = br.readLine()) != null) {
									sb.append(str);
								}
								br.close();
							} catch (IOException e1) {
								e1.printStackTrace();
							}
							help_param.setText(sb.toString());
						}
					});
				}
				grid.pack();
				item.setControl(grid);
			}
			tabFolder.pack();
		} catch (Exception e) {
			this.view_error(e, "Problems during loading default params");
		}
	}

	private void delete_xml_file() throws MSMSException {
		try {
			XtandemModelFile xml = this.get_current_xml();
			if (xml.exists()) {
				MessageBox messageBox = new MessageBox(shell, SWT.YES | SWT.NO);
				messageBox.setMessage("Deleted the model file :\n"
						+ xml.getName());
				int response = messageBox.open();
				if (response == SWT.YES) {
					xml.delete();
					this.set_model_list(xml);
				}
			}
		} catch (Exception e) {
			String message = "ERROR :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void load_value_to_xml() {
		try {
			XtandemModelFile xml = this.get_current_xml();
			this.params.load_xml(xml);
			for (Combo com : combo_field) {
				xtandem_param param = (xtandem_param) com.getData("param");
				com.select(param.getValueindex());
			}
			for (Text t : text_field) {
				xtandem_param param = (xtandem_param) t.getData("param");
				t.setText(param.getValue());
			}
			this.modelname.setText(xml.getName().replaceFirst("\\.xml$", ""));
			logger.debug("load");
		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during parsing of file");
		}

	}

	/**
	 * get the current selected X!Tandem model file
	 * 
	 * @return
	 * @throws MSMSException
	 */
	private XtandemModelFile get_current_xml() throws MSMSException {
		String filename = model_list.getItem(model_list.getSelectionIndex());
		File c = new File(conf.getXtandemModelsDirectory()
				+ System.getProperty("file.separator") + filename);
		return new XtandemModelFile(c);
	}

	private void save_value_to_xml() {
		try {
			// File xml = this.get_current_xml(name + ".xml");
			XtandemModelFile xml = conf
					.getXtandemModelFile(modelname.getText());
			logger.debug(xml.getAbsolutePath());

			int count = 0;
			for (Combo com : combo_field) {
				xtandem_param param = (xtandem_param) com.getData("param");
				param.setValue(com.getItem(com.getSelectionIndex()));
			}
			for (Text t : text_field) {
				xtandem_param param = (xtandem_param) t.getData("param");
				boolean valid = param.setValue(t.getText());
				if (!valid) {
					count++;
					t.setBackground(shell.getDisplay().getSystemColor(
							SWT.COLOR_RED));
				} else
					t.setBackground(shell.getDisplay().getSystemColor(
							SWT.COLOR_WHITE));
			}

			if (count > 0)
				throw new StopException(count + " values are wrong");
			if (xml.exists()) {
				MessageBox messageBox = new MessageBox(shell, SWT.YES | SWT.NO);
				messageBox.setMessage("The model file : " + xml.getName()
						+ " exist.\n Do you want to overwrite?");
				int reponse = messageBox.open();
				if (reponse == SWT.YES)
					params.save_to_xml(xml);
			} else
				params.save_to_xml(xml);
			logger.debug("save");

			this.set_model_list(xml);
		} catch (StopException e) {
			MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
			messageBox.setMessage(e.getMessage());
			messageBox.open();
		} catch (Exception e) {
			this.view_error(e, "Problems during save parameters");
		}
	}

	private void restore_default_value() {
		for (Combo com : combo_field) {
			xtandem_param param = (xtandem_param) com.getData("param");
			param.restoreDefault_value();
			com.select(param.getValueindex());
		}
		for (Text t : text_field) {
			xtandem_param param = (xtandem_param) t.getData("param");
			param.restoreDefault_value();
			t.setText(param.getValue());
		}
		this.modelname.setText("New model");
		logger.debug("restore");
	}

	protected void view_error(Exception e, String text_error) {
		StackTraceElement[] f = e.getStackTrace();
		String error = e.getMessage() + "\nat : " + f[0].toString() + "\n";
		MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
		messageBox.setMessage(text_error + "\n" + error);
		messageBox.open();
		e.printStackTrace();
	}
}
