package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.fdr_count;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;

public class calculation_peptide_fdr_distribution extends
		calculation_base_protein {

	private HashMap<String, fdr_count> fdrs;

	private HashMap<String, String> databases;

	private HashMap<HashSampleScan, String> peptide_view;

	private String[] xtandemNormRev = new String[] { "xtandem normal",
			" xtandem reverse" };

	public calculation_peptide_fdr_distribution() {
		super();
		databases = new HashMap<String, String>();
		fdrs = new HashMap<String, fdr_count>();
		this.peptide_view = new HashMap<HashSampleScan, String>();
		logger.info("Print peptide FDR distribution");
	}

	protected void processing_match(Match m) throws MSMSException {
		double elimite = -1;
		while (elimite > -30.1) {
			fdr_count fd;
			if (fdrs.containsKey("" + elimite))
				fd = fdrs.get("" + elimite);
			else {
				fd = new fdr_count();
				fd.setEmoins(elimite - 0.5);
				fd.setEplus(elimite);
				fdrs.put("" + elimite, fd);
			}

			String data = m.get_protein_match().getDatabase().getDatabasePath();
			if (!databases.containsKey(data))
				databases.put(data, "" + 0);
			// on ajoute deux fausse base de données pour normal/reverse de
			// X!tandem
			for (String d : xtandemNormRev) {
				if (!databases.containsKey(d))
					databases.put(d, "" + 0);
			}

			for (Peptide pep : m.get_peptide_order()) {
				if (!peptide_view.containsKey(pep.getHashSampleScan())) {
					fd.add_count(pep.get_evalue(), data);

					if (m.get_protein_match().isReversed()) {
						fd.add_count(pep.get_evalue(), xtandemNormRev[1]);
					} else {
						fd.add_count(pep.get_evalue(), xtandemNormRev[0]);
					}
				}
			}

			elimite += -0.5;
		}

		// on ajoute les peptides dans la table à ne plus tenir compte
		for (Peptide pep : m.get_peptide_order()) {
			if (!peptide_view.containsKey(pep.getHashSampleScan())) {
				peptide_view.put(pep.getHashSampleScan(), "ok");
			}
		}
	}

	public void print_peptide_fdr_distribution(OdsDocWriterInterface out)
			throws Exception {
		out.writeSheet("peptide_fdr_distrib");
		ArrayList<String> datas = this.get_databases_order();
		ArrayList<fdr_count> fds = this.get_fdr_order();
		//StringBuilder S = new StringBuilder("Peptide Evalue");
		out.writeCell("Peptide Evalue");
		for (String d : datas) {
			//S.append("\t" + d);
			out.writeCell(d);
		}
		out.writeLine();
		//S.append("\n");
		//out.write(S.toString());
		for (fdr_count fd : fds) {
			//S = new StringBuilder("");
			out.writeCell(fd.getEplus());
			//S.append(fd.getEplus());
			for (String d : datas) {
				int tmp = Integer.parseInt(databases.get(d));
				tmp = tmp + Integer.parseInt(fd.get_count_to_database(d));
				databases.put(d, "" + tmp);
				out.writeCell(tmp);
				//S.append("\t" + tmp);
			}
			//S.append("\n");
			//out.write(S.toString());
			out.writeLine();
		}
	}

	private ArrayList<fdr_count> get_fdr_order() {
		ArrayList<fdr_count> fdr_counts = new ArrayList<fdr_count>(
				fdrs.values());
		Collections.sort(fdr_counts);
		return fdr_counts;
	}

	private ArrayList<String> get_databases_order() {
		ArrayList<String> datas = new ArrayList<String>(this.databases.keySet());
		Collections.sort(datas);
		return datas;
	}
}
