package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.HashMap;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;

public class calculation_protein_fdr extends calculation_base_protein {

	private String[] reverse_database;
	private config conf;
	private float reverse;
	private float normal;
	private HashMap<HashSampleScan, String> peptide_view;

	public calculation_protein_fdr(config conf, String[] databases) {
		super();
		this.reverse_database = databases;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new HashMap<HashSampleScan, String>();
		logger.info("Calculation of protein fdr using separated reverse database");
	}

	public calculation_protein_fdr(config conf) {
		super();
		this.reverse_database = null;
		this.conf = conf;
		this.reverse = 0;
		this.normal = 0;
		this.peptide_view = new HashMap<HashSampleScan, String>();
		logger.info("Calculation of protein fdr using X!Tandem/Mascot reverse option");
	}

	protected void processing_match(Match m) throws MSMSException {
		boolean specific_peptide = true;
		for (Peptide pep : m.get_peptide_order())
			if (peptide_view.containsKey(pep.getHashSampleScan()))
				specific_peptide = false;

		if (m.get_unique_valid_peptide_number() >= conf.get_peptide_number()
				&& m.get_protein_match().get_evalue() <= conf
						.get_protein_evalue() && specific_peptide) {
			boolean rev = false;
			if (reverse_database != null) {
				for (String d : reverse_database) {
					if (m.get_protein_match().getDatabase().getDatabasePath()
							.equals(d)) {
						rev = true;
					}
				}
			} else {
				if (m.get_protein_match().isReversed()) {
					rev = true;
				}
			}
			if (rev)
				reverse++;
			else
				normal++;
		}
		// on les rajoute dans la table
		for (Peptide pep : m.get_peptide_order())
			peptide_view.put(pep.getHashSampleScan(), "ok");
	}

	public float get_protein_fdr() {
		if (normal == 0)
			return (0);
		float val = reverse / normal * 100;
		return (val);
	}
}
