/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.input.fasta;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;

/**
 * @brief parses a FASTA file
 *
 *        parse a FASTA file, extracts each sequences, look for display id in
 *        database and set sequence if there was no sequence for this display id
 *
 *        test it with wget : wget --post-file=sequences_file.fasta
 *        --user=langella --password=yourpwd
 *        http://host/proticport/sequences/fasta
 *
 * @author langella
 *
 */

public class FastaInputStreamParser {
	private static final Logger logger = Logger
			.getLogger(FastaInputStreamParser.class);
	// private static final Logger logger =
	// Logger.getLogger(FastaInputStreamParser.class);
	private FastaReader reader;

	public FastaInputStreamParser(FastaReader reader) {
		super();
		this.reader = reader;
	}

	protected void newFastaSequence(String header, String strSequence)
			throws Exception {
		this.reader.newFastaSequence(header, strSequence);
	}

	/**
	 * The parsing is done in this method
	 *
	 * @return
	 * @throws Exception
	 */
	public void parseFastaInputStream(InputStream is) throws Exception {
		logger.debug("parseFastaInputStream begin");
		InputStreamReader isr = new InputStreamReader(is, "Cp850");
		BufferedReader br = new BufferedReader(isr);

		String line = "";
		String header = "";
		StringBuilder sb = new StringBuilder();
		int maxSequenceLength = -1;
		boolean keepGoing = true;
		// line = br.readLine();
		do {
			line = line.trim(); // nice to have but probably not needed
			//logger.debug(line);
			if (line.length() != 0) {
				if (line.startsWith(">")) {
					if (sb.length() > 0) {
						// logger.debug(header + " " + sb.toString());
						newFastaSequence(header, sb.toString());
						if (maxSequenceLength < sb.length()) {
							maxSequenceLength = sb.length();
						}
						sb = new StringBuilder(maxSequenceLength);
					}
					header = line.substring(1);
				} else if (line.startsWith(";")) {
				} else {
					sb.append(line);
				}
			}
			line = br.readLine();
			if (line == null) {
				logger.debug(header + " " + sb.toString());
				newFastaSequence(header, sb.toString());
				keepGoing = false;
			}
		} while (keepGoing);
		br.close();
		isr.close();
		logger.debug("parseFastaInputStream end");
	}
}
