package fr.inra.pappso.xtandempipeline.ui.results;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.filter_print.swt_table_peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class Peptide_List_item extends CTabItem {
	private static final Logger logger = Logger
			.getLogger(Peptide_List_item.class);

	private String[] search_type = new String[] { "Sample", "Scan", "Sequence",
			"Modifs", "Specific", "Valid" };

	private window_base window;

	private Vector<Button> buttons;

	protected Table table;

	protected HashSampleScanSetProt protein;

	protected Label protein_label;

	private DataType_Base data_type;

	protected Label pep_number;

	protected Identification ident;

	protected Button cancel;

	public Peptide_List_item(window_base parent, int posi) {
		super(parent.getFolder(), SWT.CLOSE, posi);
		this.window = parent;
		buttons = new Vector<Button>();
		this.data_type = new DataType_Base(window.getConf());
		// on ajoute le contenu
		this.setControl(this.create_composite());
		for (Button but : buttons)
			but.setEnabled(false);
	}

	protected Composite create_composite() {
		Composite compo_principal = new Composite(window.getFolder(), SWT.NONE);
		GridLayout lay = new GridLayout(1, false);
		lay.verticalSpacing = 10;
		lay.horizontalSpacing = 5;
		compo_principal.setLayout(lay);

		protein_label = new Label(compo_principal, SWT.WRAP);
		protein_label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		table = new Table(compo_principal, SWT.BORDER | SWT.CHECK
				| SWT.FULL_SELECTION);
		table.setLayoutData(new GridData(GridData.FILL_BOTH));
		table.setHeaderVisible(true);
		table.setLinesVisible(false);
		table.addSelectionListener(new SelectionAdapter() {
			int action_time = 0;

			public void widgetSelected(SelectionEvent e) {
				TableItem item = (TableItem) e.item;
				Peptide pep = (Peptide) item.getData("peptide");
				try {
					if (e.detail == SWT.CHECK) {
						logger.debug("Change pep validité");
						for (Peptide pep2 : Peptide_List_item.this.ident
								.get_peptide_liste()) {

							if (pep2.getHashSampleScan() == pep
									.getHashSampleScan()) {
								pep2.setIsValid(item.getChecked());
							}
						}
						action_time = e.time;
					} else if (e.time != action_time) {
						logger.debug("view " + pep.getHashSampleScan());
						window.getControlleur().view_peptide(pep);
					}
				} catch (MSMSException e1) {
					logger.error(e1.getMessage());
					e1.printStackTrace();
				}
			}
		});

		Composite compo = new Composite(compo_principal, SWT.None);
		compo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		compo.setLayout(new GridLayout(5, false));
		Combo comb = new Combo(compo, SWT.DROP_DOWN | SWT.READ_ONLY);
		for (String se : search_type)
			comb.add(se);
		comb.select(0);
		Text t = new Text(compo, SWT.SINGLE);
		t.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Button but = new Button(compo, SWT.PUSH);
		but.setText("Search");
		but.setData("text", t);
		but.setData("combo", comb);
		but.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Text t = (Text) e.widget.getData("text");
				Combo comb = (Combo) e.widget.getData("combo");
				Peptide_List_item.this.filter_data(t.getText(), comb
						.getSelectionIndex());
				logger.debug("Search");
				Peptide_List_item.this.cancel.setEnabled(true);
			}

		});
		buttons.add(but);
		this.cancel = new Button(compo, SWT.PUSH);
		cancel.setText("Cancel");
		cancel.setData("text", t);
		cancel.setEnabled(false);
		cancel.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Text t = (Text) e.widget.getData("text");
				t.setText("");
				Peptide_List_item.this.filter_data(t.getText(), 0);
				logger.debug("cancel search");
				Peptide_List_item.this.cancel.setEnabled(false);
			}

		});
		pep_number = new Label(compo, SWT.RIGHT);
		pep_number.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		return (compo_principal);
	}

	protected void filter_data(String text, int selectionIndex) {
		logger.debug("Filter : " + text + " in "
				+ (this.search_type[selectionIndex]));
		try {
			if (!this.data_type.getIs_active())
				throw new StopException("");
			swt_table_peptide swt = new swt_table_peptide(table, this.data_type);
			swt.setProtein(protein);
			swt.setfiltercolumn(this.search_type[selectionIndex]);
			swt.setfiltervalue(text);
			ident.view_result(swt);
			pep_number.setText(table.getItemCount() + " peptides");
			// pep_number.update();
		} catch (StopException e) {
			logger.debug("No data");
		} catch (Exception e) {
			window.view_error(e, "Problems with filtering peptide list");
		}
	}

	public void load_data(Identification ident, HashSampleScanSetProt protein,
			DataType_Base data_type) {
		try {
			for (Button but : buttons)
				but.setEnabled(true);
			this.protein = protein;
			this.data_type = data_type;
			this.ident = ident;
			swt_table_peptide swt = new swt_table_peptide(table, data_type);
			swt.setProtein(protein);
			ident.view_result(swt);
			Match match = (Match) protein.getGrpProtMatch();
			this.protein_label.setText(match.get_protein_match()
					.get_description());
			pep_number.setText(table.getItemCount() + " peptides");
			this.setText(match.get_protein_match().get_protein_accession());
		} catch (StopException e) {
			logger.debug("shell not open");
		} catch (Exception e) {
			window.view_error(e, "Problems with drawing peptide list");
		}
	}

	public HashSampleScanSetProt getProtein() {
		return protein;
	}

	public void setProtein(HashSampleScanSetProt protein) {
		this.protein = protein;
	}

	public DataType_Base getData_type() {
		return data_type;
	}

	public void setData_type(DataType_Base data_type) {
		this.data_type = data_type;
	}

	public Identification getIdent() {
		return ident;
	}

	public void setIdent(Identification ident) {
		this.ident = ident;
	}
}
